#include "fx_lib.h"

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <assert.h>





/* Create two fifos with radom names */
void create_fifos (char *cmd_fifo, char *rep_fifo)
{
	char *status_c;
	int status_i;

#define CMD_FIFO "/tmp/cmd-XXXXXX"
#define REP_FIFO "/tmp/rep-XXXXXX"

	strcpy (cmd_fifo, CMD_FIFO);
	status_c = mktemp (cmd_fifo);
	assert (status_c != NULL);
	status_i = mkfifo (cmd_fifo, 0600);
	assert (status_i == 0);
	
	strcpy (rep_fifo, REP_FIFO);
	status_c = mktemp (rep_fifo);
	assert (status_c != NULL);
	status_i = mkfifo (rep_fifo, 0600);
	assert (status_i == 0);
}


void
register_in_fiddle (int nd_pid, const char *cmd_fifo, const char *rep_fifo)
{
	/* Launch a grandchild, which will register into fiddle and
	   request the addition of the new debugger */

	if (fork() == 0) {
		if (fork() == 0) {
			/* Grandchild */
		tkout_t *tk; 
		puts ("fx_initialize()");
#if defined f2m
		fx_initialize (NULL, NULL, START_SERVER);
#else
		fx_initialize ();
#endif
		puts ("fx_add_debugger()");
#if defined f2m || f1m
		tk = fx_add_debugger ("localhost", nd_pid, 0, cmd_fifo, rep_fifo);
#else
		tk = fx_add_debugger (pid, 0, cmd_fifo, rep_fifo);
#endif
		f0s_tkout_delete (tk);
		usleep(5e5);
/* 		tk = fx_break (1, NULL, -1, "main"); */
/* 		f0s_tkout_delete (tk); */

		puts ("fx_terminate()");
#if defined f2m
		fx_terminate (0);
#else
		fx_terminate ();
#endif
		puts ("exit()");
		exit (0);
		}
		else
			/* Child: terminates immediately */
			exit(0);
	}

	/* Father wait for son to die */
	wait(NULL);
}



/* Should receive the name of the program to launch 
 * as a command line argument
 */
main (int argc, char *argv[])
{
	struct stat sbuf;
	char *file_name;
	char cmd_fifo[40];
	char rep_fifo[40];
	int status_i;
	int pid;
	int fd;
	
/* 	assert (argc  == 2); */

	/* Use 'Infinito' as the default test file */
	file_name = (argc < 2) ? "/home/jml/Infinito" : argv[1];
	assert (file_name != NULL);

	/* Check if 'file' is a regular file with execution permission */
	status_i = stat (file_name, &sbuf);
	if (status_i < 0) return 0;
	if (!(S_ISREG(sbuf.st_mode))) return 0; /* BUG -- must check
											   permissions too */ 

	/* Create two fifos with radom names */
	create_fifos (cmd_fifo, rep_fifo);
		
	/* Get debugger pid */
	pid = getpid ();
	assert (pid != -1);


	/* Printing data */
/* 	printf ("LAUNCHER: cld_pid = %d\n", pid); */
/* 	printf ("LAUNCHER: tp_pid = %d\n", 0); */
/* 	printf ("LAUNCHER: cmd_channel = %s\n", cmd_fifo); */
/* 	printf ("LAUNCHER: rep_channel = %s\n", rep_fifo); */
/* 	printf ("add-debugger %d %d %s %s\n", pid, 0, cmd_fifo, rep_fifo); */

	register_in_fiddle (pid, cmd_fifo, rep_fifo);

	/* Redirect STDIN */
	close (0);
	fd = open (cmd_fifo, O_RDONLY);
	assert (fd == 0);

	/* Redirect STDOUT */
	close (1);
	fd = open (rep_fifo, O_WRONLY);
	assert (fd == 1);

	/* Redirect STDERR */
	dup2 (1, 2);

	/* Execute CLD */
	fcntl(1, F_SETFL, O_APPEND);
	if (setvbuf(stdout, NULL, _IONBF, 0) < 0)
		return -1;
	execlp ("gdb", "gdb", "--fullname", "-q", file_name, NULL);
}
