/* *****************************************************************
 *
 *  $Id: f0m_queues.h,v 1.2 2002/08/14 01:30:24 jml Exp $
 *
 *  Copyright (c) 1999 by Dep.Informatica of the FCT/UNL
 *  Time-stamp: <02/08/14 00:59:15 jml>
 *
 *  Joao Lourenco <Joao.Lourenco@di.fct.unl.pt>
 *
 *  Parallel and Distributed Processing Group
 *  Departamento de Informatica
 *  Faculdade de Ciencias e Tecnologia
 *  Universidade Nova de Lisboa
 *
 * -----------------------------------------------------------------
 *
 *  Description: Prototypes for the enqueue requests and dequeue
 *				 replies services
 *
 *		f0m_queues_init ()		-- Initialization
 *		f0m_queues_terminate ()	-- Termination
 *		f0m_enqueue_request ()	-- Enqueue a request
 *		f0m_dequeue_request ()	-- Dequeue a request
 *		f0m_enqueue_reply ()		-- Enqueue a reply
 *		f0m_dequeue_reply ()		-- Dequeue a reply
 *
 * -----------------------------------------------------------------
 */
#ifndef __F1_QUEUES_H
#define __F1_QUEUES_H


#include "f0s_tkout.h"
#include "f0m_tkin.h"
#include "glob.h"



/* ID for system services, not associated to a apecific task (yet),
 * such as 'f0m_tids()' and 'fi_attach()' */
#define NO_TID -1


typedef struct request_s request_t;
struct request_s {
	const tkin_t  *tk;
	unsigned long  id;
	int			   tid;
};


typedef struct reply_s reply_t;
struct reply_s {
	const tkout_t *tk;
	unsigned long  id;
	int			   tid;
};



__BEGIN_DECLS

/* -----------------------------------------------------------------
 * Function:    int f0m_queues_init (void)
 *
 * Description: Initialize queues for requests and replies
 *
 * Returns:      0 if OK
 *				-1 on error
 */
int f0m_queues_init (void);

/* -----------------------------------------------------------------
 * Function:    int f0m_queues_terminate (void)
 *
 * Description: Terminate queues for requests and replies
 *
 * Returns:      0 if OK
 *				-1 on error
 */
int f0m_queues_terminate (void);


/* -----------------------------------------------------------------
 * Function:    int f0m_enqueue_request (int tid, const tkin_t *req)
 *
 * Description: 'tid'	The (symbolic) id of the target process
 *				'req'	Token with the request code and arguments
 *
 * Returns:     The request ID
 */
int f0m_enqueue_request (int tid, const tkin_t *tk);


/* -----------------------------------------------------------------
 * Function:    request_t *req f0m_dequeue_request (int tid)
 *
 * Description: 'tid'	The (symbolic) id of the target process
 *
 * Returns:     A structure 'request_t'
 */
request_t *f0m_dequeue_request (int tid);


/* -----------------------------------------------------------------
 * Function:    int repid = f0m_enqueue_reply (int tid, int id,
 *											  const tkout_t *rep)
 *
 * Description: 'tid'	The (symbolic) id of the target process
 *				'rep'	Token with the reply code and arguments
 *
 * Returns:     The status
 */
int f0m_enqueue_reply (int tid, int id, const tkout_t *tk);


/* -----------------------------------------------------------------
 * Function:    tkout_t *tk = f0m_dequeue_reply (int reqid)
 *
 * Description: Get the reply to the request with ID 'reqid'
 *
 * Returns:     Pointer to the reply ('tkout_t') structure
 */
tkout_t *f0m_dequeue_reply (int reqid);



__END_DECLS

#endif /* __F1_QUEUES_H */
