/* *****************************************************************
 *
 *  $Id: f0m_tkin.h,v 1.7 2002/09/10 15:54:32 jml Exp $
 *
 *  Copyright (c) 1999 by Dep.Informatica of the FCT/UNL
 *  Time-stamp: <02/08/13 11:42:45 jml>
 *
 *  Joao Lourenco <Joao.Lourenco@di.fct.unl.pt>
 *
 *  Parallel and Distributed Processing Group
 *  Departamento de Informatica
 *  Faculdade de Ciencias e Tecnologia
 *  Universidade Nova de Lisboa
 *
 * -----------------------------------------------------------------
 *
 *  Description: Description of the 'Input Token', that contains the
 *				 command code and its arguments, and the request
 *				 identifier.
 *
 *				 Prototypes of the services associated with the
 *				 'Input Token'.
 *
 *		f0m_tkin_new ():			Create a new input token
 *		f0m_tkin_delete ():		Delete an input token and its contents
 *
 * -----------------------------------------------------------------
 */
#ifndef __F1_TKIN_H
#define __F1_TKIN_H

#include <stdarg.h>
#include "glob.h"
#include "f0m_cmd_codes.h"

typedef struct req_clients_s req_clients_t;
struct req_clients_s {
	const char *options;
};

typedef struct req_tids_s req_tids_t;
struct req_tids_s {
	const char *options;
};

typedef struct req_kill_s req_kill_t;
struct req_kill_s {
	/* No arguments */
};


typedef struct req_attach_s req_attach_t;
struct req_attach_s {
	int			tp_pid;
	const char *host;			/* Used only in Level 1 */
};

typedef struct req_detach_s req_detach_t;
struct req_detach_s {
	/* No arguments */
};

typedef struct req_symbol_file_s req_symbol_file_t;
struct req_symbol_file_s {
	const char *file;
	const char *host;			/* Used only in Level 1 */
};

typedef struct req_file_s req_file_t;
struct req_file_s {
	const char *file;
	const char *host;			/* Used only in Level 1 */
};

typedef struct req_run_s req_run_t;
struct req_run_s {
	const char *args;
};

typedef struct req_step_s req_step_t;
struct req_step_s {
	/* No arguments */
};

typedef struct req_next_s req_next_t;
struct req_next_s {
	/* No arguments */
};

typedef struct req_continue_s req_continue_t;
struct req_continue_s {
	/* No arguments */
};

typedef struct req_finish_s req_finish_t;
struct req_finish_s {
	/* No arguments */
};

typedef struct req_signal_s req_signal_t;
struct req_signal_s {
	int tid;
	int signum;
};

typedef struct req_break_s req_break_t;
struct req_break_s {
	const char *file;
	int         line;
	const char *function;
};

typedef struct req_delete_s req_delete_t;
struct req_delete_s {
	int bpid;
};

typedef struct req_set_variable_s req_set_variable_t;
struct req_set_variable_s {
	const char *variable;
	const char *expression;
};

typedef struct req_evaluate_s req_evaluate_t;
struct req_evaluate_s {
	const char *expression;
};

typedef struct req_display_s req_display_t;
struct req_display_s {
	const char *expression;
};

typedef struct req_undisplay_s req_undisplay_t;
struct req_undisplay_s {
	int dispid;
};

typedef struct req_call_s req_call_t;
struct req_call_s {
	const char *expression;
};

typedef struct req_list_s req_list_t;
struct req_list_s {
	const char *file;
	int         start_line;
	const char *function;
	int         n_lines;
};

typedef struct req_tty_s req_tty_t;
struct req_tty_s {
	const char *file;
};

typedef struct req_up_s req_up_t;
struct req_up_s {
	/* No arguments */
};

typedef struct req_down_s req_down_t;
struct req_down_s {
	/* No arguments */
};

typedef struct req_frame_s req_frame_t;
struct req_frame_s {
	int frame;
};

typedef struct req_info_locals_s req_info_locals_t;
struct req_info_locals_s {
	const char *var;
	const char *value;
};

typedef struct req_info_line_s req_info_line_t;
struct req_info_line_s {
	const char *file;
	int         line;
	const char *function;
};

typedef struct req_info_program_s req_info_program_t;
struct req_info_program_s {
	/* No arguments */
};

typedef struct req_info_stack_s req_info_stack_t;
struct req_info_stack_s {
	/* No arguments */
};

typedef struct req_info_break_s req_info_break_t;
struct req_info_break_s {
	/* No arguments */
};

typedef struct req_info_display_s req_info_display_t;
struct req_info_display_s {
	/* No arguments */
};

typedef struct req_thread_s req_thread_t;
struct req_thread_s {
	int threadID;
};

typedef struct req_info_threads_s req_info_threads_t;
struct req_info_threads_s {
	/* No arguments */
};

typedef struct req_sendto_s req_sendto_t;
struct req_sendto_s {
	const char *command;
};

typedef struct req_terminate_s req_terminate_t;
struct req_terminate_s {
	int halt;
};

typedef struct req_register_client_s req_register_client_t;
struct req_register_client_s {
	int pid;
	const char *filename;
	const char *hostaddr;
};

typedef struct req_add_debugger_s req_add_debugger_t;
struct req_add_debugger_s {
	int dbg_pid;
	int tp_pid;
	const char *host;
	char *cmd_channel;
	char *rep_channel;
};


typedef struct tkin_s tkin_t;
struct tkin_s {
/* 	int		reqid; */				/* The request ID */
	int		tid;				/* Task ID of target process */
	cmd_code_t	code;				/* Service code */
	union {
		req_tids_t			r_tids_clients;
		req_kill_t			r_kill;
		req_add_debugger_t	r_add_debugger;
		req_attach_t		r_attach;
		req_detach_t		r_detach;
		req_symbol_file_t	r_symbol_file;
		req_file_t			r_file;
		req_run_t			r_run;
		req_step_t			r_step;
		req_next_t			r_next;
		req_continue_t		r_continue;
		req_finish_t		r_finish;
		req_signal_t		r_signal;
		req_break_t			r_break;
		req_delete_t		r_delete;
		req_set_variable_t	r_set_variable;
		req_evaluate_t		r_evaluate;
		req_display_t		r_display;
		req_undisplay_t		r_undisplay;
		req_call_t			r_call;
		req_list_t			r_list;
		req_tty_t			r_tty;
		req_up_t			r_up;
		req_down_t			r_down;
		req_frame_t			r_frame;
		req_info_locals_t	r_info_locals;
		req_info_line_t		r_info_line;
		req_info_program_t	r_info_program;
		req_info_stack_t	r_info_stack;
		req_info_break_t	r_info_break;
		req_info_display_t	r_info_display;
		req_thread_t		r_thread;
		req_info_threads_t	r_info_threads;
		req_sendto_t		r_sendto;
		req_terminate_t		r_terminate;
		req_register_client_t	r_register_client;
	} data;
};



__BEGIN_DECLS

/* -----------------------------------------------------------------
 * Function:    tkin_t *tkin = f0m_tkin_new (in tid, cmd_code_t code, ...)
 *
 * Description: Create a new 'Input Token'
 *				'code'	The code for the service
 *				'...'	Arguments to service 'code'
 *
 * Returns:     Pointer to the new token, allocated from the heap
 */
tkin_t *f0m_tkin_new (int tid, cmd_code_t code, ...);


/* -----------------------------------------------------------------
 * Function:    void f0m_tkin_delete (tkin_t *tk)
 *
 * Description: Delete an 'Input Token' and itds its contents
 *
 * Returns:     []
 */
void f0m_tkin_delete (tkin_t *tk);



__END_DECLS


#endif /* __F1_TKIN_H */
