/* *****************************************************************
 *
 *  $Id: f0s_cmd_gen.c,v 1.1.1.1 2002/04/28 22:06:50 jml Exp $
 *
 *  Copyright (c) 1999 by Dep.Informatica of the FCT/UNL
 *  Time-stamp: <00/11/16 20:21:51 jml>
 *
 *  Joao Lourenco <Joao.Lourenco@di.fct.unl.pt>
 *
 *  Parallel and Distributed Processing Group
 *  Departamento de Informatica
 *  Faculdade de Ciencias e Tecnologia
 *  Universidade Nova de Lisboa
 *
 * -----------------------------------------------------------------
 *
 *  Description:
 *
 * -----------------------------------------------------------------
 */

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include "f0s_cmd_gen.h"
#include "config.h"


#define sprinta1(s,f,x)	sprintf ((s) + strlen(s), (f), (x))
#define sprinta2(s,f,x,y)	sprintf ((s) + strlen(s), (f), (x), (y))


/* -----------------------------------------------------------------
 * Function:    char *cmd_str = f0s_gen_command (IN cmd_code_t cmd_code,
 &													IN ...)
 *
 * Description: Given a command code, generate the corresponding
 *				command string.
 *
 * Returns:     A pointer to the command string.
 */
char *f0s_gen_command (cmd_code_t cmd_code, ...)
{
#define MAX_STR 512
	va_list ap;
	char cmd_str[MAX_STR];

	va_start (ap, cmd_code);
	switch (cmd_code) {
	case CMD_KILL: {
		sprintf (cmd_str, "kill");
		break;
	}
	case CMD_ATTACH: {
		int pid = va_arg(ap, int);
		sprintf (cmd_str, "attach %d", pid);
		break;
	}
	case CMD_DETACH: {
		sprintf (cmd_str, "detach");
		break;
	}
	case CMD_SYMBOL_FILE: {
		const char *file = va_arg(ap, const char *);
		sprintf (cmd_str, "symbol-file %s", file);
		break;
	}
	case CMD_FILE: {
		const char *file = va_arg(ap, const char *);
		sprintf (cmd_str, "file %s", file);
		break;
	}
	case CMD_RUN: {
		const char *args = va_arg(ap, const char *);
		sprintf (cmd_str, "run %s", (args == NULL) ? "" : args);
		break;
	}
	case CMD_STEP: {
		sprintf (cmd_str, "step");
		break;
	}
	case CMD_NEXT: {
		sprintf (cmd_str, "next");
		break;
	}
	case CMD_CONTINUE: {
		sprintf (cmd_str, "continue");
		break;
	}
	case CMD_FINISH: {
		sprintf (cmd_str, "finish");
		break;
	}
	case CMD_BREAK: {
		const char *file = va_arg(ap, const char *);
		int line = va_arg(ap, int);
		const char *function = va_arg(ap, const char *);
		sprintf (cmd_str, "break ");
		if (file != NULL)
			sprinta1 (cmd_str, "%s:", file);
		if (function != NULL)
			sprinta1 (cmd_str, "%s", function);
		else
			sprinta1 (cmd_str, "%d", line);
		break;
	}
	case CMD_DELETE: {
		int bpid = va_arg(ap, int);
		sprintf (cmd_str, "delete %d", bpid);
		break;
	}
	case CMD_SET_VARIABLE: {
		const char *variable = va_arg(ap, const char *);
		const char *expression = va_arg(ap, const char *);
		sprintf (cmd_str, "set variable %s = %s", variable, expression);
		break;
	}
	case CMD_EVALUATE: {
		const char *expression = va_arg(ap, const char *);
		sprintf (cmd_str, "print %s", expression);
		break;
	}
	case CMD_INFO_LOCALS: {
		sprintf (cmd_str, "info locals");
		break;
	}
	case CMD_INFO_ARGS: {
		sprintf (cmd_str, "info args");
		break;
	}
	case CMD_DISPLAY: {
		const char *expression = va_arg(ap, const char *);
		sprintf (cmd_str, "display %s", expression);
		break;
	}
	case CMD_UNDISPLAY: {
		int dispid = va_arg(ap, int);
		sprintf (cmd_str, "undisplay %d", dispid);
		break;
	}
	case CMD_CALL: {
		const char *function_and_args = va_arg(ap, const char *);
		sprintf (cmd_str, "call %s", function_and_args);
		break;
	}
	case CMD_LIST: {
/* The number of lines defaults to 10 */
#define N_LINES 10
		const char *file = va_arg(ap, const char *);
		int start_line = va_arg(ap, int);
		const char *function = va_arg(ap, const char *);
		int n_lines = va_arg(ap, int);
		if (n_lines == -1)
			n_lines = N_LINES;
		sprintf (cmd_str, "set listsize %d\nlist ", n_lines);
		if (file != NULL)
			sprinta1 (cmd_str, "%s:", file);
		if (function != NULL)
			sprinta1 (cmd_str, "%s", function);
		else
			sprinta2 (cmd_str, "%d,%d",
					  start_line, n_lines + start_line - 1);
		break;
	}
	case CMD_TTY: {
		const char *file = va_arg(ap, const char *);
		sprintf (cmd_str, "tty %s", file);
		break;
	}
	case CMD_INFO_LINE: {
		const char *file = va_arg(ap, const char *);
		int line = va_arg(ap, int);
		const char *function = va_arg(ap, const char *);
		sprintf (cmd_str, "info line ");
		if (file != NULL)
			sprinta1 (cmd_str, "%s:", file);
		if (function != NULL)
			sprinta1 (cmd_str, "%s", function);
		if (line != -1)
			sprinta1 (cmd_str, "%d", line);
		break;
	}
	case CMD_INFO_PROGRAM: {
		sprintf (cmd_str, "info program");
		break;
	}
	case CMD_INFO_STACK: {
		sprintf (cmd_str, "info stack");
		break;
	}
	case CMD_UP: {
		sprintf (cmd_str, "up");
		break;
	}
	case CMD_DOWN: {
		sprintf (cmd_str, "down");
		break;
	}
	case CMD_FRAME: {
		int frame = va_arg(ap, int);
		sprintf (cmd_str, "frame %d", frame);
		break;
	}
	case CMD_INFO_BREAK: {
		sprintf (cmd_str, "info break");
		break;
	}
	case CMD_INFO_DISPLAY: {
		sprintf (cmd_str, "info display");
		break;
	}
	case CMD_THREAD: {
		int threadID = va_arg(ap, int);
		sprintf (cmd_str, "thread %d", threadID);
		break;
	}
	case CMD_INFO_THREADS: {
		sprintf (cmd_str, "info threads");
		break;
	}
	default:
		return NULL;
	}
	va_end (ap);
	return strdup (cmd_str);
}
