/* *****************************************************************
 *
 *  $Id: f0s_gdb_regex.c,v 1.3 2002/09/10 15:54:32 jml Exp $
 *
 *  Copyright (c) 1999 by Dep.Informatica of the FCT/UNL
 *  Time-stamp: <02/07/05 15:38:54 jml>
 *
 *  Joao Lourenco <Joao.Lourenco@di.fct.unl.pt>
 *
 *  Parallel and Distributed Processing Group
 *  Departamento de Informatica
 *  Faculdade de Ciencias e Tecnologia
 *  Universidade Nova de Lisboa
 *
 * -----------------------------------------------------------------
 *
 *  Description:
 *
 * -----------------------------------------------------------------
 */

#include <stdlib.h>
#include "f0s_out_codes.h"
#include "f0s_regex.h"
#include "f0s_gdb_regex.h"
#include "config.h"


/* =================================================================
  		.		--> any character but '\n'
		*		--> any character 0 to n times
		+		--> any character 1 to n times
		?		--> any character 0 or 1 time
		^		--> begin of line or NOT following character
		$		--> end of line or '$'
		\\w		--> character '0..9a..zA...Z'
		\\W		--> character NOT '0..9a..zA...Z'
		\\<		--> word begin (word is composed of 0..9a..zA...Z)
		\\>		--> word end (word is composed of 0..9a..zA...Z)
		\\b		--> word bound
		\\B		--> not word bound
		\\|		--> means OR
		\\`		--> begin buffer
		\\\'	--> end buffer
   ================================================================= */




/* This structure contain the regular expressions that match the
 * output of GDB to each of the supported commands. */
PatRec reply_patterns[] = {
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_KILL
	 *++++++++++++++++++++++++++++++++++++++++
> Kill the program being debugged? (y or n) y
	 *		2 = The relevant string
	 */
	{CMD_KILL,
	 R_KILL,
/* 	 "\\(.*\n\\)*\ */
/* \\(\ */
/* Kill the program being debugged? (y or n) y\ */
/* \\)\ */
/* \n\\(.*\n\\)*", */
	 NULL,
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_ATTACH
	 *++++++++++++++++++++++++++++++++++++++++
> Attaching to process 764
	 *		2 = The relevant string
	 *		3 = The target process identifier
	 */
	{CMD_ATTACH,
	 R_ATTACH,
/* 	 "\\(.*\n\\)*\ */
/* \\(\ */
/* Attaching to [^ ]+ \\([0-9]+\\)\ */
/* \\)\ */
/* \n\\(.*\n\\)*", */
	 NULL,
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_DETACH
	 *++++++++++++++++++++++++++++++++++++++++
> Detaching from program: PID 764
	 *		2 = The relevant string
	 *		3 = The target process identifier
	 */
	{CMD_DETACH,
	 R_DETACH,
/* 	 "\\(.*\n\\)*\ */
/* \\(\ */
/* Detaching from program: .* \\([0-9]+\\)\ */
/* \\)\ */
/* \n\\(.*\n\\)*", */
	 NULL,
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_SYMBOL_FILE
	 *++++++++++++++++++++++++++++++++++++++++
> Reading symbols from Infinito.c...done.	 
	 *		2 = The relevant string
	 *		3 = The file name
	 */
	{CMD_SYMBOL_FILE,
	 R_SYMBOL_FILE,
/* 	 "\\(.*\n\\)*\ */
/* \\(\ */
/* Reading symbols from \\(.*\\)\\.\\.\\.done\\.\ */
/* \\)\ */
/* \n\\(.*\n\\)*", */
	 NULL,
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_FILE
	 *++++++++++++++++++++++++++++++++++++++++
> Reading symbols from Infinito.c...done.	 
	 *		2 = The relevant string
	 *		3 = The file name
	 */
	{CMD_FILE,
	 R_FILE,
/* 	 "\\(.*\n\\)*\ */
/* \\(\ */
/* Reading symbols from \\(.*\\)\\.\\.\\.done\\.\ */
/* \\)\ */
/* \n\\(.*\n\\)*", */
	 NULL,
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_RUN
	 *++++++++++++++++++++++++++++++++++++++++
> Reading symbols from Infinito.c...done.	 
	 *		2 = The relevant string
	 *		3 = The file name
	 */
	{CMD_RUN,
	 R_RUN,
	 "\\(.*\n\\)*\
\\(\
Starting program: \\(.*\\)\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_STEP
	 *++++++++++++++++++++++++++++++++++++++++
	 */
	{CMD_STEP,
	 R_STEP,
	 NULL,
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_NEXT
	 *++++++++++++++++++++++++++++++++++++++++
	 */
	{CMD_NEXT,
	 R_NEXT,
	 NULL,
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_NEXT
	 *++++++++++++++++++++++++++++++++++++++++
	 */
	{CMD_CONTINUE,
	 R_CONTINUE,
	 NULL,
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_FINISH
	 *++++++++++++++++++++++++++++++++++++++++
0x8048590 in main () at Infinito.c:42
yy/home/jml/work/ndam/DAM/console/Infinito.c:42:434:beg:0x8048590
Value returned is $1 = {x = 1234, y = 56.7890015, z = "teste"}

> Run till exit from #0  sfake (str=0x8048742 "teste") at Infinito.c:16
> 0x80485e0 in main () at Infinito.c:43
> \032\032/home/jml/work/ndam/DAM/console/Infinito.c:43:455:beg:0x80485e0
> Value returned is $2 = {x = 1234, y = 56.7890015, z = "teste"}
	 *		2 = The relevant string
	 *		3 = The stopping address
	 *		4 = The stopping function name
	 *	    5 = The stopping function arguments
	 *	    6 = The stopping file name
	 *	    7 = The stopping line number
	 *	    8 = The debugging variable number
	 *	    9 = The value returned
	 */
	{CMD_FINISH,
	 R_FINISH,
	 "\\(.*\n\\)*\
\\(\
\\(0x[^ ]+\\) in \\([^ ]+\\) (\\(.*\\)) \
at \\(.*\\):\\([0-9]+\\)\n\
.+\n\
Value returned is \\$\\([0-9]+\\) = {\\(.*\\)}\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_CALL
	 * ++++++++++++++++++++++++++++++++++++++++
> $11 = 0x8048731 "a test with char *"
> $6 = "test"
> $5 = 1073773012
> $4 = {
>   x = 1073815520, 
>   y = 2.01054573, 
>   z = "teste"
> }
	 *		2 = The relevant string (set of lines)
	 *
	 * ATTENTION: This returns the same as CMD_EVALUATE, i.e. R_EVALUATE
	 */
	{CMD_CALL,
	 R_EVALUATE,
	 "\\([^$].*\n\\)*\
\\(\
\\(\
\\(\\$[0-9]+.+{\n\\(.+\n\\)*[ \t]*}\n\\)\\|\
\\(\\$[0-9]+.+[^{]\n\\)\
\\)+\
\\)\
\\(\\)",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_CALL
	 * ++++++++++++++++++++++++++++++++++++++++
	 *		2 = The relevant string (set of lines)
	 *
	 * ATTENTION: For 'void function()' returns nothing
	 */
	{CMD_CALL,
	 R_CALL_VOID,
	 NULL,
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_BREAK
	 *++++++++++++++++++++++++++++++++++++++++
> Breakpoint 2 at 0x8048590: file Infinito.c, line 33.
	 *		2 = The relevant string
	 *		3 = The breakpoint number
	 *		4 = The memory address
	 *		5 = The file name
	 *		6 = The line number
	 */
	{CMD_BREAK,
	 R_BREAK,
	 "\\(.*\n\\)*\
\\(\
Breakpoint \\([0-9]+\\) at \\(0x[^:]+\\): \
file \\([^,]+\\), line \\([0-9]+\\)\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_BREAK_NO_SOURCE
	 *++++++++++++++++++++++++++++++++++++++++
> Breakpoint 3 at 0x804848c
	 *		2 = The relevant string
	 *		3 = The breakpoint number
	 *		4 = The memory address
	 */
	{CMD_BREAK,
	 R_BREAK_NO_SOURCE,
	 "\\(.*\n\\)*\
\\(\
Breakpoint \\([0-9]+\\) at \\(0x[^:]+\\)\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_DELETE
	 *++++++++++++++++++++++++++++++++++++++++
	 */
	{CMD_DELETE,
	 R_DELETE,
	 NULL,
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_SET_VARIABLE
	 *++++++++++++++++++++++++++++++++++++++++
	 */
	{CMD_SET_VARIABLE,
	 R_SET_VARIABLE,
	 NULL,
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_EVALUATE
	 * ++++++++++++++++++++++++++++++++++++++++
> $11 = 0x8048731 "a test with char *"
> $6 = "test"
> $5 = 1073773012
> $4 = {
>   x = 1073815520, 
>   y = 2.01054573, 
>   z = "teste"
> }
	 *		2 = The relevant string (set of lines)
	 */
	{CMD_EVALUATE,
	 R_EVALUATE,
	 "\\([^$].*\n\\)*\
\\(\
\\(\
\\(\\$[0-9]+.+{\n\\(.+\n\\)*[ \t]*}\n\\)\\|\
\\(\\$[0-9]+.+[^{]\n\\)\
\\)+\
\\)\
\\(\\)",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_INFO_LOCALS
	 * ++++++++++++++++++++++++++++++++++++++++
> buf1 = "@"
> buf2 = 0x5ed <Address 0x5ed out of bounds>
> buf2 = 0x8048731 "a test with char *"
> x = 1073833048
> y = 0
> z = 7
> d = 2.1750645637913717
> s1 = {
>   x = 1073815520, 
>   y = 2.01054573, 
>   z = "teste"
> }
	 *		2 = The relevant string (set of lines)
	 */
	{CMD_INFO_LOCALS,
	 R_INFO_LOCALS,
	 "\\(\\)*\
\\(\
\\(\
\\(.+ = +{\n\\(.+\n\\)*[ \t]*}\n\\)\\|\
\\(.+ = +.*[^{]\n\\)\
\\)+\
\\)\
\\(\\)",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_INFO_ARGS
	 * ++++++++++++++++++++++++++++++++++++++++
> buf1 = "@"
> buf2 = 0x5ed <Address 0x5ed out of bounds>
> buf2 = 0x8048731 "a test with char *"
> x = 1073833048
> y = 0
> z = 7
> d = 2.1750645637913717
> s1 = {
>   x = 1073815520, 
>   y = 2.01054573, 
>   z = "teste"
> }
	 *		2 = The relevant string (set of lines)
	 */
	{CMD_INFO_ARGS,
	 R_INFO_LOCALS,
	 "\\(\\)*\
\\(\
\\(\
\\(.+ = +{\n\\(.+\n\\)*[ \t]*}\n\\)\\|\
\\(.+ = +.*[^{]\n\\)\
\\)+\
\\)\
\\(\\)",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_DISPLAY
	 *++++++++++++++++++++++++++++++++++++++++
> 3: z + 5 = 8
> 2: y == 2 = 0
> 1: x = 0
	 *		2 = The relevant string (set of lines)
	 */
	{CMD_DISPLAY,
	 R_DISPLAY_EXPRESSION,
	 NULL,
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_UNDISPLAY
	 *++++++++++++++++++++++++++++++++++++++++
	 */
	{CMD_UNDISPLAY,
	 R_UNDISPLAY,
	 NULL,
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_LIST
	 *++++++++++++++++++++++++++++++++++++++++
> 29      void hello()
> 30      {
> 31          puts ("Hello from Infinito");
> 32      }
	 *		2 = The relevant string (set of lines)
	 */
	{CMD_LIST,
	 R_LIST,
	 "\\([^0-9].*\n\\)*\
\\(\
\\([0-9]+.*\n\\)+\
\\)\
\\(.*\n\\)*",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_TTY
	 *++++++++++++++++++++++++++++++++++++++++
	 */
	{CMD_TTY,
	 R_TTY,
	 NULL,
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_INFO_LINE
	 *++++++++++++++++++++++++++++++++++++++++
> Line 30 of "Infinito.c" starts at address 0x8048548 <hello+1>
>    and ends at 0x804854b <hello+3>.
	 *		2 = The relevant string
	 *		3 = The line number
	 *		4 = The file name
	 *		5 = The start address
	 *		6 = The start function name
	 *		8 = The offset for start function 
	 *		9 = The end address
	 *		10 = The end function name
	 *	   12 = The offset for end function 
	 */
	{CMD_INFO_LINE,
	 R_INFO_LINE,
	 "\\(.*\n\\)*\
\\(\
Line \\([0-9]+\\) of \"\\([^\"]+\\)\"[\n ] *\
starts at address \\(0x[^ ]+\\) <\\([^+>]+\\)\\(\\+\\([0-9]+\\)\\)?>[ \n] *\
and ends at \\(0x[^ ]+\\) <\\([^+>]+\\)\\(\\+\\([0-9]+\\)\\)?>\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_INFO_LINE_NO_CODE
	 *++++++++++++++++++++++++++++++++++++++++
> Line 29 of "Infinito.c" is at address 0x8048548 <hello> but contains no code.
	 *		2 = The relevant string
	 *		3 = The line number
	 *		4 = The file name
	 *		5 = The start address
	 *		6 = The start function name
	 *		8 = The offset for start function 
	 */
	{CMD_INFO_LINE,
	 R_INFO_LINE_NO_CODE,
	 "\\(.*\n\\)*\
\\(\
Line \\([0-9]+\\) of \"\\([^\"]+\\)\" \
is at address \\(0x[^ ]+\\) <\\([^+>]+\\)\\(\\+\\([0-9]+\\)\\)?> \
but contains no code\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_INFO_PROGRAM
	 *++++++++++++++++++++++++++++++++++++++++
>         Using the running image of child process 1398.
>>>>> or
>         Using the running image of child Pid 1398.
> Program stopped at 0x8048564.
> It stopped at breakpoint 1.
>>>>> or
> It stopped with signal SIGINT, Interrupt.
>>>>> or
> It stopped after being stepped.
     *		2 = The relevant string
	 *		3 = The target process PID
	 *		4 = The memory address
	 *		5 = Why did it stop (e.g. "breakpoint", "signal")
	 *	    6 = The breakpoint number or signal name
	 */
	{CMD_INFO_PROGRAM,
	 R_INFO_PROGRAM,
	 "\\(.*\n\\)*\
\\(\
[ \t]+Using the running image of [^0-9]+\\([0-9]+\\)\\.\
Program stopped at \\(0x[0-9a-fA-F]+\\)\\.\
It stopped [^ ]+ \\(breakpoint\\|signal\\|being stepped\\) ?\\(.*\\)\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_INFO_STACK
	 *++++++++++++++++++++++++++++++++++++++++
> #0  sfake (str=0x80486f2 "teste") at Infinito.c:16
> #1  0x80485c6 in main () at Infinito.c:42
	 *		2 = The relevant string
	 */
	{CMD_INFO_STACK,
	 R_INFO_STACK,
	 "\\([^#].*\n\\)*\
\\(\
\\(#.*\n\\)+\
\\)\
\\([^#].*\n\\)*",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_UP
	 *++++++++++++++++++++++++++++++++++++++++
> #0  sfake (str=0x80486f2 "teste") at Infinito.c:16
>>>>> or
> #0  0x80485c6 in main () at Infinito.c:42
	 *		2 = The relevant string
	 */
	{CMD_UP,
	 R_INFO_STACK,
	 "\\([^#].*\n\\)*\
\\(\
\\(#.*\n\\)+\
\\)\
\\([^#].*\n\\)*",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_DOWN
	 *++++++++++++++++++++++++++++++++++++++++
> #0  sfake (str=0x80486f2 "teste") at Infinito.c:16
>>>>> or
> #0  0x80485c6 in main () at Infinito.c:42
	 *		2 = The relevant string
	 */
	{CMD_DOWN,
	 R_INFO_STACK,
	 "\\([^#].*\n\\)*\
\\(\
\\(#.*\n\\)+\
\\)\
\\([^#].*\n\\)*",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_FRAME
	 *++++++++++++++++++++++++++++++++++++++++
> #0  sfake (str=0x80486f2 "teste") at Infinito.c:16
>>>>> or
> #0  0x80485c6 in main () at Infinito.c:42
	 *		2 = The relevant string
	 */
	{CMD_FRAME,
	 R_INFO_STACK,
	 "\\([^#].*\n\\)*\
\\(\
\\(#.*\n\\)+\
\\)\
\\([^#].*\n\\)*",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_INFO_BREAK
	 *++++++++++++++++++++++++++++++++++++++++
> Num Type           Disp Enb Address    What
> 1   breakpoint     keep y   0x08048593 in main at Infinito.c:38
>         breakpoint already hit 1 time
> 2   hw breakpoint  keep n   0x0804850c in sfake at Infinito.c:16
>         stop only if str == 0
> 5   breakpoint     keep y   0x080485cc in main at Infinito.c:42
> 6   breakpoint     del  y   0x0804854e in fake at Infinito.c:26
> 7   breakpoint     keep y   0x400514da in strcpy
>                                        at ../sysdeps/generic/strcpy.c:30
> 8   breakpoint     keep y   0x40096f1d in xdrmem_create at xdr_mem.c:82
> 9   breakpoint     keep y   0x40041b36 in vprintf at vprintf.c:32
> 10  breakpoint     keep y   0x40053e92 in str_append at argz-replace.c:29
> 11  hw watchpoint  keep y   str == 0
> 13  hw watchpoint  keep y   str
	 *		2 = The relevant string (set of lines)
	 */
	{CMD_INFO_BREAK,
	 R_INFO_BREAK,
	 "\\([^0-9].*\n\\)*\
\\(\
\\(\
\\([0-9]+.*\n\\)\\|\
\\([ \t]+\\(breakpoint already\\|stop only\\|ignore next\\|at\\).+\\)\
\\)+\
\\)",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_INFO_DISPLAY
	 * ++++++++++++++++++++++++++++++++++++++++
> Auto-display expressions now in effect:
> Num Enb Expression
> 3:   y  z + 5 = 8
> 2:   y  y == 2 = 0
> 1:   n  x = 0
	 *		2 = The relevant string (set of lines)
	 */
	{CMD_INFO_DISPLAY,
	 R_INFO_DISPLAY,
	 "\\([^0-9].*\n\\)*\
\\(\
\\(\
\\([0-9]+:.+\n\\)\
\\)+\
\\)\
\\(\\)",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_INFO_THREADS
	 * ++++++++++++++++++++++++++++++++++++++++
> * 3 Thread 11337  process (arg=0x80486c1) at ex1.c:13
>   2 Thread 11335  __pthread_sighandler (sig=-1073744160) at pthread.c:340
>   1 Thread 11336  0x4008a2de in __select ()
	 *		2 = The relevant string (set of lines)
	 */
	{CMD_INFO_THREADS,
	 R_INFO_THREADS,
	 "\\(..[^0-9].*\n\\)*\
\\(\
\\(..[0-9]+ Thread.*\n\\)+\
\\)\
\\(..[^0-9].*\n\\)*",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_INFO_THREADS
	 * ++++++++++++++++++++++++++++++++++++++++
	 * For the case when "info threads" retunrs an empty string
	 *		2 = The relevant string (empty string)
	 */
	{CMD_INFO_THREADS,
	 R_INFO_THREADS,
	 "",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_THREAD
	 *++++++++++++++++++++++++++++++++++++++++
	 */
	{CMD_THREAD,
	 R_THREAD,
	 NULL,
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_SENDTO
	 *++++++++++++++++++++++++++++++++++++++++
	 */
	{CMD_SENDTO,
	 R_SENDTO,
	 "\\(\
\\(\
\\(.*\n\\)*\
\\)\
\\)",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	LAST ONE
	 *++++++++++++++++++++++++++++++++++++++++ */
	{-1, -1, NULL, NULL}
};



/* This structure contain the regular expressions necessary to host
 * items in a multi-line GDB output (e.g. info stack)*/
PatRec line_patterns[] = {
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_INFO_STACK_LINE
	 * ++++++++++++++++++++++++++++++++++++++++
> #0  sfake (str=0x80486f2 "teste") at Infinito.c:16
>>>>> or
> #1  0x80485c6 in main () at Infinito.c:42
>>>>> or
> #0  0x4008a2de in __select ()
	 *		1 = The frame number
	 *		3 = The frame address (optional)
	 *		4 = The function name
	 *		5 = The function arguments
	 *		8 = File name
	 *		9 = The line number
	 */
	{CMD_INFO_STACK,
	 R_INFO_STACK,
	 "\
#\\([0-9]+\\)[ \t]+\\(\\(0x[^ ]+\\) in \\)?\\([^ ]+\\) (\\(\\(.+, \n\\)*.*\\))\
\\( at \\([^:]+\\):\\([0-9]+\\)\\)?",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_INFO_BREAK_LINE_1
	 *++++++++++++++++++++++++++++++++++++++++
> Num Type           Disp Enb Address    What
> 1   breakpoint     keep y   0x08048593 in main at Infinito.c:38
> 7   breakpoint     keep y   0x400514da in strcpy
	 *		1 = The breakpoint number
	 *		2 = Type (e.g. "breakpoint", "hw watchpoint")
	 *		3 = "keep" for permanent or "del" for temporary breakpoints
	 *		4 = "y" for enabled or "n" for disabled
	 *		5 = The address
	 *	    6 = Function name
	 *	    8 = File name
	 *	    9 = Line number
	 */
	{CMD_INFO_BREAK,
	 R_INFO_BREAK,
	 "\\([0-9]+\\)[ \t]+\\(.*breakpoint\\)[ \t]+\\([^ ]+\\)[ \t]+\\(y\\|n\\)[ \t]+\\(0x[^ ]+\\)[ \t]+in \\([^ \t\n]+\\)\\([ \t]+at \\([^:]+\\):\\([0-9]+\\)\\)?",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_INFO_BREAK_LINE_2
	 *++++++++++++++++++++++++++++++++++++++++
> Num Type           Disp Enb Address    What
> 11  hw watchpoint  keep y   str == 0
> 13  hw watchpoint  keep y   str
>         breakpoint already hit 4 times
>         ignore next 2 hits
	 *		1 = The breakpoint number
	 *		2 = Type (e.g. "breakpoint", "hw watchpoint")
	 *		3 = "keep" for permanent or "del" for temporary breakpoints
	 *		4 = "y" for enabled or "n" for disabled
	 *		5 = The watchpoint condition
	 */
	{CMD_INFO_BREAK,
	 R_INFO_WATCHPOINT,
	 "\\([0-9]+\\)[ \t]+\\(.*watchpoint\\)[ \t]+\\([^ ]+\\)[ \t]+\\(y\\|n\\)[ \t]+\\([^0].*\\)",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_INFO_BREAK_LINE_3
	 *  This is a modifier to the previous breakpoint data
	 *++++++++++++++++++++++++++++++++++++++++
> Num Type           Disp Enb Address    What
>         stop only if str == 0
>         breakpoint already hit 1 time
>         ignore next 1 hits
>                                        at ../sysdeps/generic/strcpy.c:30
	 *		3 = Conditional breapoint expression
	 *		5 = Hit counter
	 *		7 = Ignore counter
	 *	    9 = File name
	 *	    10 = Line number
	 */
	{CMD_INFO_BREAK,
	 R_INFO_BREAK_WATCH_DATA,
	 "[ \t]+\\\(\
\\(stop only if \\(.*\\)\\)\\|\
\\(breakpoint already hit \\([0-9]+\\).*\\)\\|\
\\(ignore next \\([0-9]+\\).*\\)\\|\
\\(at \\([^:]+\\):\\([0-9]+\\).*\\)\
\\)",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	R_INFO_LOCALS_ONE_LINE
	 * ++++++++++++++++++++++++++++++++++++++++
> buf1 = "@"
> buf2 = 0x5ed <Address 0x5ed out of bounds>
> buf2 = 0x8048731 "a test with char *"
> x = 1073833048
> y = 0
> z = 7
> d = 2.1750645637913717
	 *		1 = Variable name
	 *		2 = Variable value
	 */
	{CMD_INFO_LOCALS,
	 R_INFO_LOCALS_ONE_LINE,
	 "\\(^[^ ]+\\) = \\(.*[^{]$\\)",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_INFO_LOCALS_MANY_{START,MIDDLE,END}
	 * ++++++++++++++++++++++++++++++++++++++++
> s1 = {
>   x = 1073815520, 
>   y = 2.01054573, 
>   z = "teste"
> }
	 *		1 = Variable name
	 *		2 = Variable value
	 */
	{CMD_INFO_LOCALS,
	 R_INFO_LOCALS_MANY_START,
	 "\\(.+\\) = {",
	 NULL},
	{CMD_INFO_LOCALS,
	 R_INFO_LOCALS_MANY_MIDDLE,
	 "\\(.+ = .+\\)",
	 NULL},
	{CMD_INFO_LOCALS,
	 R_INFO_LOCALS_MANY_END,
	 "[ \t]*}$",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_EVALUATE_ONE_LINE
	 * ++++++++++++++++++++++++++++++++++++++++
> $11 = 0x8048731 "a test with char *"
> $6 = "test"
> $5 = 1073773012
	 *		1 = The dbugger's variable number
	 *		2 = The value
	 */
	{CMD_EVALUATE,
	 R_EVALUATE_ONE_LINE,
	 "\\$\\([0-9]+\\) = \\(.*[^{]$\\)",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_EVALUATE_MANY_{START,MIDDLE,END}
	 * ++++++++++++++++++++++++++++++++++++++++
> $4 = {
>   x = 1073815520, 
>   y = 2.01054573, 
>   z = "teste"
> }
	 *		1 = The display number
	 *		2 = The value
	 */
	{CMD_EVALUATE,
	 R_EVALUATE_MANY_START,
	 "\\$\\([0-9]+\\) = {",
	 NULL},
	{CMD_EVALUATE,
	 R_EVALUATE_MANY_MIDDLE,
	 "\\(.+ = .+\\)",
	 NULL},
	{CMD_EVALUATE,
	 R_EVALUATE_MANY_END,
	 "[ \t]*}$",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_DISPLAY_ONE_LINE
	 * ++++++++++++++++++++++++++++++++++++++++
> 3:  z + 5 = 8
> 2:  y == 2 = 0
> 1:  x = 0
	 *		1 = The display number
	 *		2 = Th expression
	 *		3 = The value
	 */
	{CMD_DISPLAY,
	 R_DISPLAY_ONE_LINE,
	 "\\([0-9]+\\):[ \t]+\\(.+\\) = \\([^{=][^=]*\\)",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_DISPLAY_MANY_{START,MIDDLE,END}
	 * ++++++++++++++++++++++++++++++++++++++++
> 1: s1 = {
>      x = 1073815520,
>      y = 2.01054573,
>      z = "test"
>    }
	 *		1 = The display number
	 *		2 = Th expression
	 *		3 = The value
	 */
	{CMD_DISPLAY,
	 R_DISPLAY_MANY_START,
	 "\\([0-9]+\\): \\(.+\\) = {",
	 NULL},
	{CMD_DISPLAY,
	 R_DISPLAY_MANY_MIDDLE,
	 "\\(.+ = .+\\)",
	 NULL},
	{CMD_DISPLAY,
	 R_DISPLAY_MANY_END,
	 "[ \t]*}",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_LIST
	 *++++++++++++++++++++++++++++++++++++++++
> 29      void hello()
> 30      {
> 31          puts ("Hello from Infinito");
> 32      }
	 *		1 = The line number
	 *		2 = The line text
	 */
	{CMD_LIST,
	 R_LIST,
	 "\\([0-9]+\\)\t\\(.*\\)",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_INFO_DISPLAY
	 * ++++++++++++++++++++++++++++++++++++++++
> **********Auto-display expressions now in effect:
> **********Num Enb Expression
> 3:  y  z + 5
> 2:  y  y == 2
> 1:  n  x
	 *		1 = The display number
	 *		2 = Enabled/disabled
	 *		3 = Th expression
	 */
	{CMD_INFO_DISPLAY,
	 R_INFO_DISPLAY,
	 "\\([0-9]+\\):[ \t]+\\(y\\|n\\)[ \t]+\\(.+\\)",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_INFO_THREADS
	 * ++++++++++++++++++++++++++++++++++++++++
> * 3 Thread 11337  process (arg=0x80486c1) at ex1.c:13
>   2 Thread 11335  __pthread_sighandler (sig=-1073744160) at pthread.c:340
>   1 Thread 11336  0x4008a2de in __select ()
	 *		1 = Active (flag)
	 *		2 = The thread ID
	 *		3 = The thread PID
	 *		5 = Stack address
	 *		6 = Function name
	 *		7 = Function arguments
	 *	    9 = File name
	 *	   10 = Line number
	 */
	{0,
	 R_INFO_THREADS,
	 "\
\\(\\*\\| \\) \\([0-9]+\\) Thread \\([0-9]+\\)  \
\\(\\(0x[^ ]+\\) in \\)?\
\\([^ ]+\\) (\\(.*\\))\
\\( at \\([^:]+\\):\\([0-9]+\\)\\)?\
",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_NEW_THREAD
	 * ++++++++++++++++++++++++++++++++++++++++
> [New Thread 2051 (LWP 15077)]
	 *		1 = The thread number
	 */
	{0,
	 R_NEW_THREAD,
	 "\
\\[New Thread \\([0-9]+\\).*\\]\
",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	LAST ONE
	 *++++++++++++++++++++++++++++++++++++++++ */
	{-1, -1, NULL, NULL}
};



/* This structure contains the regular patterns of a special case of the
 * ALWAYS patterns defined just below, but due to its importance it
 * deserves a special treatment */
PatRec position_patterns[] = {
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_POSITION
	 * ++++++++++++++++++++++++++++++++++++++++
> \032\032/home/jml/work/DAMS-NEW/Pdbg/tests/Infinito.c:19:178:beg:0x8048559
	 *		2 = The relevant string
	 *		3 = The file name
	 *		4 = The line
	 *		5 = char_pos
	 *		6 = "middle" or "beg"
	 *		7 = The frame address
	 */
	{0,
	 P_POSITION,
	 "\\(.*\n\\)*\
\\(\
\032\032\\([^\n]+\\):\\([0-9]+\\):\\([0-9]+\\):\\([^:]+\\):\\(.+\\)\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	LAST ONE
	 *++++++++++++++++++++++++++++++++++++++++ */
	{-1, -1, NULL, NULL}
};



/* This structure contain the regular expressions with output that can 
 * show up at any time, and that should be oob matced.  This
 * does not include the output on error conditions! */
PatRec oob_patterns[] = {
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_DISPLAY
	 *++++++++++++++++++++++++++++++++++++++++
> 3: z + 5 = 8
> 2: y == 2 = 0
> 1: x = 0
	 *		2 = The relevant string (set of lines)
	 */
	{0,
	 O_DISPLAY,
	 "\\([^0-9].*\n\\)*\
\\(\
\\(\
\\([0-9]+:.+{\n\\(.+\n\\)*[ \t]*}\n\\)\\|\
\\([0-9]+:.+[^{]\n\\)\
\\)+\
\\)\
\\(\\)",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_BREAKPOINT_HITED
	 * ++++++++++++++++++++++++++++++++++++++++
> Breakpoint 1, main (argc=1, argv=0xbffff954) at target.c:47
	 *		2 = The relevant string
	 *		3 = The breakpoint number
	 *		4 = The function name
	 *		5 = The function arguments
	 *		7 = File name
	 *		8 = The line number
	 */
	{0,
	 O_BPHIT,
	 "\\(.*\n\\)*\
\\(\
Breakpoint \\([0-9]+\\), \\([^ ]+\\) (\\(\\(.+, \n\\)*.*\\))\
 at \\([^:]+\\):\\([0-9]+\\)\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_SIGNAL
	 * ++++++++++++++++++++++++++++++++++++++++
> Program received signal SIGINT, Interrupt.
	 *		2 = The relevant string
	 *		3 = The signal name
	 */
	{0,
	 O_SIGNAL,
	 "\\(.*\n\\)*\
\\(\
Program received signal \\([A-Z]+\\), Interrupt\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_NEW_THREAD
	 * ++++++++++++++++++++++++++++++++++++++++
> [New Thread 2051 (LWP 15077)]
	 *		2 = The relevant string (set of lines)
	 */
	{0,
	 O_NEW_THREAD,
	 "\\(.*\n\\)*\
\\(\
\\(\\[New Thread \\([0-9]+\\).*\\]\n\\)+\
\\)\
\\(.*\n\\)*",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	O_SWITCH_TO_THREAD
	 * ++++++++++++++++++++++++++++++++++++++++
> [Switching to Thread 1697]
	 *		2 = The relevant string (set of lines)
	 *		3 = The thread PID
	 */
	{0,
	 O_SWITCH_TO_THREAD,
	 "\\(.*\n\\)*\
\\(\
\\[Switching to Thread \\([0-9]+\\).*\\]\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	LAST ONE
	 *++++++++++++++++++++++++++++++++++++++++ */
	{-1, -1, NULL, NULL}
};


/* This structure contain the regular expressions that match the
 * output on error conditions. */
PatRec error_patterns[] = {
	{0,
	 E_NOT_RUN,
	 "\\(.*\n\\)*\
\\(\
The program.*is not being run\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_NO_FRAME_SELECTED,
	 "\\(.*\n\\)*\
\\(\
No frame selected\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_NO_PROCESS,
	 "\\(.*\n\\)*\
\\(\
ptrace: No such process\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_NOT_PERMITTED,
	 "\\(.*\n\\)*\
\\(\
ptrace: Operation not permitted\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_NO_SYMBOL,
	 "\\(.*\n\\)*\
\\(\
No symbol \"\\([^\"]+\\)\" in current context\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_BOTTOM,
	 "\\(.*\n\\)*\
\\(\
Bottom (i.e., innermost) frame selected; you cannot go down\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_TOP,
	 "\\(.*\n\\)*\
\\(\
Initial frame selected; you cannot go up\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_INVALID_EXPRESSION,
	 "\\(.*\n\\)*\
\\(\
.*: Invalid expression\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_NO_SYMBOLS,
	 "\\(.*\n\\)*\
\\(\
(no debugging symbols found).*\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_NO_SYMBOL_TABLE,
	 "\\(.*\n\\)*\
\\(\
No symbol table is loaded.  Use the \"file\" command.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_NO_DEF_SOURCE_FILE,
	 "\\(.*\n\\)*\
\\(\
Can't find a default source file\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_INVALID_BPID,
	 "\\(.*\n\\)*\
\\(\
No breakpoint number \\([0-9]+\\)\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_INVALID_LINE_NUMBER,
	 "\\(.*\n\\)*\
\\(\
No line \\([0-9]+\\) in file \"\\([^\"]+\\)\"\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_FUNCTION_NOT_DEFINED,
	 "\\(.*\n\\)*\
\\(\
Function \"\\([^\"]+\\)\" not defined\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_LINE_OUT_OF_RANGE,
	 "\\(.*\n\\)*\
\\(\
Line number \\([0-9]+\\) is out of range for \"\\([^\"]+\\)\"\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_LINE_OUT_OF_RANGE,
	 "\\(.*\n\\)*\
\\(\
Line number \\([0-9]+\\) out of range; \\([^ ]+\\) has \\([0-9]+\\) lines\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_NO_BOUNDS,
	 "\\(.*\n\\)*\
\\(\
Cannot find bounds of current function\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_INVALID_SYMBOL,
	 "\\(.*\n\\)*\
\\(\
No symbol \"\\(.*\\)\" in current context\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_INVALID_DISPLAY_NUMBER,
	 "\\(.*\n\\)*\
\\(\
No display number \\([0-9]+\\)\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_NO_DISPLAY_EXPRESSION,
	 "\\(.*\n\\)*\
\\(\
There are no auto-display expressions now\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_PARSE_ERROR,
	 "\\(.*\n\\)*\
\\(\
A parse error in expression.*\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_TOO_FEW_ARGS,
	 "\\(.*\n\\)*\
\\(\
.* few arguments in function call\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_NO_STACK,
	 "\\(.*\n\\)*\
\\(\
No stack\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_NO_BREAK_OR_WATCH,
	 "\\(.*\n\\)*\
\\(\
No breakpoints or watchpoints\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_JUNK_AT_END_OF_LINE,
	 "\\(.*\n\\)*\
\\(\
Junk at end of line specification: \\(.+\\)\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_NO_LINE_NUM_INFO,
	 "\\(.*\n\\)*\
\\(\
No line number information available\\.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_INVALID_THREAD_ID,
	 "\\(.*\n\\)*\
\\(\
Thread ID \\([0-9]+\\) not known\\.\
\\).*\n.*\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_NO_THREAD_ID,
	 "\\(.*\n\\)*\
\\(\
Please specify a thread ID.*\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_NO_THREAD_COMMAND,
	 "\\(.*\n\\)*\
\\(\
Please specify a command following the thread ID list\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	{0,
	 E_SIGSEGV,
	 "\\(.*\n\\)*\
\\(\
Program received signal SIGSEGV, Segmentation fault.\
\\)\
\n\\(.*\n\\)*",
	 NULL},
	/* ++++++++++++++++++++++++++++++++++++++++
	 *	LAST ONE
	 *++++++++++++++++++++++++++++++++++++++++ */
	{-1, -1, NULL, NULL}
};



PatRec *patterns[] = {
	reply_patterns,
	line_patterns,
	position_patterns,
	oob_patterns,
	error_patterns,
	NULL
};

