/* *****************************************************************
 *
 *  $Id: f0s_parse.c,v 1.1.1.1 2002/04/28 22:06:52 jml Exp $
 *
 *  Copyright (c) 1999 by Dep.Informatica of the FCT/UNL
 *  Time-stamp: <02/02/19 17:20:44 jml>
 *
 *  Joao Lourenco <Joao.Lourenco@di.fct.unl.pt>
 *
 *  Parallel and Distributed Processing Group
 *  Departamento de Informatica
 *  Faculdade de Ciencias e Tecnologia
 *  Universidade Nova de Lisboa
 *
 * -----------------------------------------------------------------
 *
 *  Description:
 *
 * -----------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "chain.h"
#include "msg.h"
#include "f0s_tkout.h"
#include "f0s_error.h"
#include "f0s_out_codes.h"
#include "f0s_pcre_file.h"
#include "f0s_parse.h"
#include "config.h"



#define MIN(a, b)  (((a) < (b)) ? (a) : (b))

typedef struct {
  int *bounds,size;
} pcre_result;

#define INIT_PCRE_RESULT(a) { \
	a->bounds=(int *)malloc(sizeof(int)*(max_patterns+1)*3); \
	a->size=(max_patterns+1)*3; \
}

PRIVATE int intialized = 0;


PRIVATE char *mystrndup (const char *str, int max)
{
	char c;
	char *s;
	int len = strlen (str);
	int n = MIN(len, max);
	c = str[n];
	((char *)str)[n] = '\0';
	s = strdup (str);
	((char *)str)[n] = c;
	return s;
}


PRIVATE char *mystrcat (char *str1, char *str2)
{
	char *newstr = malloc (strlen (str1) + strlen (str2) + 1);
	strcpy (newstr, str1);
	strcat (newstr, str2);
	free (str1);
	return newstr;
}



/* #################################################################
 *		PRIVATE functions
 * ################################################################# */



/* -----------------------------------------------------------------
 * Function:    int match = try_match (cmd_code_t code,
 *									   const int start,
 *									   const PatRec *pat,
 *									   const char *string,
 *									   OUT struct re_registers *regs)
 *
 * Description: Tries to match the 'string' against the regular_expression
 *
 * Returns:     The number of characters matched, or -1 if no pattern
 *				was found to match the string.
 */
PRIVATE int try_match (cmd_code_t code, const int start, const PatRec *pat,
					  const char *string, pcre_result *res)
{
	int m;
	int pos = -1;

	if (start > 0) {
		for (pos = 0;
			 pat[pos].cmdcode != CMD_EOLIST  &&  pat[pos].outcode != start;
			 pos++)
			;
		if (pat[pos].cmdcode == -1)
			return E_UNKNOWN_CMD_CODE;
	}
	
	for (++pos;  pat[pos].cmdcode != CMD_EOLIST;  pos++) {
		if (pat[pos].cmdcode != 0 &&  pat[pos].cmdcode != code)
			continue;			/* Does not apply. Try next pattern. */
		if (pat[pos].pat == NULL)
			return -pat[pos].outcode; /* No regex for this command */

		// Old code
		//m = re_match(pat[pos].buf, string, strlen (string), 0, regs);
		//if (m == -2 ) 			/* Match error - failure stack overflow */
		/*msg_pid_fatal ("\
xxgdb error: regular expression matching failed (failure stack overflow)\n");
                  if (m != -1)
			return pat[pos].outcode; /* A match was found. */
		/* End Old Code */

		/* New code */
		INIT_PCRE_RESULT(res);
     		m = pcre_exec(pat[pos].pat, //compiled pattern
			      pat[pos].extra, //extra information obtained through pcre_extra
			      string, //subject string
			      strlen(string), //size of subject string
			      0, //start at offset 0
			      0, //default options
			      res->bounds, //result vector
			      res->size); //size of result vector

		if (m < -1) {
		  /* MISSING: ERROR HANDLING */
		  assert(m>=-1);
		}
		if (m > -1) {  //match found
		  return pat[pos].outcode;
		}
		
		/* End New Code */
	}
	/* No matches found */
	return E_UNKNOWN_CMD_CODE;
}


/* -----------------------------------------------------------------
 * Function:    int n = extract_int (int tk_number,
 *									 const char *string,
 *									 const struct re_registers *regs)
 *
 * Description: Extract an integer from token 'tk_number' of 'regs' in 
 *				'string'.
 *
 * Returns:     The integer
 */
PRIVATE int extract_int (int tk_number, const char *string,
						const pcre_result *res)
{
	int start, len;
	int val = -1;

	start = res->bounds[tk_number*2];
	len = res->bounds[tk_number*2+1] - start;
	if (len > 0  &&  len <= strlen (string)) {
		char *s = mystrndup (&string[start], len);
		if (s != NULL) {
			sscanf (s, "%d", &val);
			free (s);
		}
	}
	return val;
}


/* -----------------------------------------------------------------
 * Function:    void *p = extract_pointer (int tk_number,
 *									       const char *string,
 *									       const struct re_registers *regs)
 *
 * Description: Extract a pointer from token 'tk_number' of 'regs' in 
 *				'string'.
 *
 * Returns:     The pointer
 */
PRIVATE void *extract_pointer (int tk_number, const char *string,
							  const pcre_result *res)
{
	int start, len;
	unsigned val = 0;

	start = res->bounds[tk_number*2];
	len = res->bounds[tk_number*2+1] - start;
	if (len > 0  &&  len <= strlen (string)) {
		char *s = mystrndup (&string[start], len);
		if (s != NULL) {
			sscanf (s, "%x", &val);
			free (s);
		}
	}
	return (void *)val;
}


/* -----------------------------------------------------------------
 * Function:    char *p = extract_string (int tk_number,
 *									      const char *string,
 *									      const struct re_registers *regs)
 *
 * Description: Extract an char from token 'tk_number' of 'regs' in 
 *				'string'.
 *
 * Returns:     The pointer
 */
PRIVATE char *extract_string (int tk_number, const char *string,
							  const pcre_result *res)
{
	int start, len;
	char *val = NULL;

	start = res->bounds[tk_number*2];
	len = res->bounds[tk_number*2+1] - start;
	if (len > 0  &&  len <= strlen (string))
		val = mystrndup (&string[start], len);
	else
		val = strdup ("");
	return val;
}



/* ================================================================= */
/* ================================================================= */
/* ================================================================= */

/* -----------------------------------------------------------------
 * Function:    PRIVATE void *line_data = extract_line (code_t code,
 *										const char *line,
 *										coid *last,
 *										const struct re_registers *regs)
 *
 * Description: Extract the data from a line
 *
 * Returns:     A new structure allocated from the heap  with the data
 *				    extracted from the line.
 */
PRIVATE void *extract_line (code_t code, const char *line, void *last,
						   const pcre_result *res)
{
	void *dline = NULL;
	switch (code) {
	case R_INFO_STACK: {
		rframe_t *f = malloc (sizeof (*f));
		if (f == NULL) break;	/* Memory error */
		f->frame = extract_int (1, line, res);
		f->address = extract_pointer (3, line, res);
		f->function = extract_string (4, line, res);
		f->arguments = extract_string (5, line, res);
		f->file = extract_string (8, line, res);
		if (strlen (f->file) == 0) {
			free ((void *)f->file);
			f->file = NULL;
			f->line = -1;
		}
		else 
			f->line = extract_int (9, line, res);
		dline = (void *)f;
		break;
	}
	case R_INFO_BREAK: {
		char *a;
		rinfobreak_t *f = malloc (sizeof (*f));
		if (f == NULL) break;	/* Memory error */
		f->bpid = extract_int (1, line, res);
		f->type = extract_string (2, line, res);
		f->disp = extract_string (3, line, res);
		a = extract_string (4, line, res);
		if (a != NULL  &&  strlen (a) == 1) {
			switch (a[0]) {
			case 'y': f->enabled = 1; break;
			case 'n': f->enabled = 0; break;
			default:
				msg_pid_write ("extract_line()::R_INFO_BREAK, invalid 'enabled' field value = %c", a[0]);
			}
			free (a);
		}
		else
			msg_pid_write ("extract_line()::R_INFO_BREAK, invalid 'enabled' field length");
		f->address = extract_pointer (5, line, res);
		f->function = extract_string (6, line, res);
		f->file = extract_string (8, line, res);
		f->line = extract_int (9, line, res);
		f->condition = NULL;
		f->hit_counter = -1;
		f->ign_counter = -1;
		dline = (void *)f;
		break;
	}
	case R_INFO_WATCHPOINT: {
		char *a;
		rinfobreak_t *f = malloc (sizeof (*f));
		if (f == NULL) break;	/* Memory error */
		f->bpid = extract_int (1, line, res);
		f->type = extract_string (2, line, res);
		f->disp = extract_string (3, line, res);
		a = extract_string (4, line, res);
		if (a!= NULL  &&  strlen (a) == 1)
			switch (a[0]) {
			case 'y': f->enabled = 1; break;
			case 'n': f->enabled = 0; break;
			default:
				msg_pid_write ("extract_line()::R_INFO_BREAK, invalid 'enabled' field value = %c", a[0]);
			}
		else
			msg_pid_write ("extract_line()::R_INFO_BREAK, invalid 'enabled' field length = %d", strlen (a));
		free (a);
		f->address = (void *)0;
		f->function = NULL;
		f->file = NULL;
		f->line = -1;
		f->condition = extract_string (5, line, res);
		f->hit_counter = -1;
		f->ign_counter = -1;
		dline = (void *)f;
		break;
	}	
	case R_INFO_BREAK_WATCH_DATA: {
		rinfobreak_t *b = (rinfobreak_t *)last;
		if (b == NULL) break;	/* Memory error */
		if (b->condition == NULL)
			b->condition = extract_string (3, line, res);
		if (b->hit_counter == -1)
			b->hit_counter = extract_int (5, line, res);
		if (b->ign_counter == -1) 
			b->ign_counter = extract_int (7, line, res);
		if (b->file == NULL) {
			char *file = extract_string (9, line, res);
			if (file != NULL) {
				b->file = file;
				b->line = extract_int (10, line, res);
			}
		}
		dline = NULL;
		break;
	}
	case R_INFO_LOCALS_ONE_LINE: {
		rinfolocals_t *d = malloc (sizeof (*d));
		if (d == NULL) break;	/* Memory error */
		d->var = extract_string (1, line, res);
		d->value = extract_string (2, line, res);
		dline = (void *)d;
		break;
	}
	case R_INFO_LOCALS_MANY_START: {
		rinfolocals_t *d = malloc (sizeof (*d));
		if (d == NULL) break;	/* Memory error */
		d->var = extract_string (1, line, res);
		d->value = strdup ("{");
		dline = (void *)d;
		break;
	}
	case R_INFO_LOCALS_MANY_MIDDLE: {
		rinfolocals_t *d = (rinfolocals_t *)last;
		char *v = extract_string (1, line, res);
		if (v != NULL) {
			if (strlen (v) != 0)
				d->value = mystrcat ((char *)d->value, "\n");
			d->value = mystrcat ((char *)d->value, "\t");
			d->value = mystrcat ((char *)d->value, v);
			free (v);
		}
		dline = NULL;
		break;
	}
	case R_INFO_LOCALS_MANY_END: {
		rinfolocals_t *d = (rinfolocals_t *)last;
		d->value = mystrcat ((char *)d->value, "\n\t}");
		dline = NULL;
		break;
	}
	case R_EVALUATE_ONE_LINE: {
		revaluate_t *d = malloc (sizeof (*d));
		if (d == NULL) break;	/* Memory error */
		d->dbg_var = extract_int (1, line, res);
		d->value = extract_string (2, line, res);
		dline = (void *)d;
		break;
	}
	case R_EVALUATE_MANY_START: {
		revaluate_t *d = malloc (sizeof (*d));
		if (d == NULL) break;	/* Memory error */
		d->dbg_var = extract_int (1, line, res);
		d->value = strdup ("{");
		dline = (void *)d;
		break;
	}
	case R_EVALUATE_MANY_MIDDLE: {
		revaluate_t *d = (revaluate_t *)last;
		char *v = extract_string (1, line, res);
		if (v != NULL) {
			if (strlen (v) != 0)
				d->value = mystrcat ((char *)d->value, "\n");
			d->value = mystrcat ((char *)d->value, "\t");
			d->value = mystrcat ((char *)d->value, v);
			free (v);
		}
		dline = NULL;
		break;
	}
	case R_EVALUATE_MANY_END: {
		revaluate_t *d = (revaluate_t *)last;
		d->value = mystrcat ((char *)d->value, "\n\t}");
		dline = NULL;
		break;
	}
	case R_DISPLAY_ONE_LINE: {
		rdisplay_t *d = malloc (sizeof (*d));
		if (d == NULL) break;	/* Memory error */
		d->dispid = extract_int (1, line, res);
		d->expression = extract_string (2, line, res);
		d->value = extract_string (3, line, res);
		d->enabled = -1;
		dline = (void *)d;
		break;
	}
	case R_DISPLAY_MANY_START: {
		rdisplay_t *d = malloc (sizeof (*d));
		if (d == NULL) break;	/* Memory error */
		d->dispid = extract_int (1, line, res);
		d->expression = extract_string (2, line, res);
		d->value = strdup ("{");
		d->enabled = -1;
		dline = (void *)d;
		break;
	}
	case R_DISPLAY_MANY_MIDDLE: {
		rdisplay_t *b = (rdisplay_t *)last;
		char *v = extract_string (1, line, res);
		if (v != NULL) {
			if (strlen (v) != 0)
				b->value = mystrcat ((char *)b->value, "\n");
			b->value = mystrcat ((char *)b->value, "\t");
			b->value = mystrcat ((char *)b->value, v);
			free (v);
		}
		dline = NULL;
		break;
	}
	case R_DISPLAY_MANY_END: {
		rdisplay_t *b = (rdisplay_t *)last;
		b->value = mystrcat ((char *)b->value, "\n\t}");
		dline = NULL;
		break;
	}
	case R_LIST: {
		rlist_t *d = malloc (sizeof (*d));
		if (d == NULL) break;	/* Memory error */
		d->line_number = extract_int (1, line, res);
		d->line_text = extract_string (2, line, res);
		dline = (void *)d;
		break;
	}
	case R_INFO_DISPLAY: {
		char *a;
		rdisplay_t *d = malloc (sizeof (*d));
		if (d == NULL) break;	/* Memory error */
		d->dispid = extract_int (1, line, res);
		a = extract_string (2, line, res);
		if (a!= NULL  &&  strlen (a) == 1)
			switch (a[0]) {
			case 'y': d->enabled = 1; break;
			case 'n': d->enabled = 0; break;
			default:
				msg_pid_write ("extract_line()::R_INFO_DISPLAY, invalid 'enabled' field value = %c", a[0]);
			}
		else
			msg_pid_write ("extract_line()::R_INFO_DISPLAY, invalid 'enabled' field length = %d", strlen (a));
		free (a);
		d->expression = extract_string (3, line, res);
		d->value = NULL;
		dline = (void *)d;
		break;
	}
	case R_INFO_THREADS: {
		char *star;
		const char *starp;
		rinfothreads_t *d = malloc (sizeof (*d));
		if (d == NULL) break;	/* Memory error */
		star = extract_string (1, line, res);
		assert (star != NULL);
		starp = star;
		assert (*starp == '*'  ||  *starp == ' ');
		d->active = (*starp == '*');
		d->th_id = extract_int (2, line, res);
		d->tp_pid = extract_int (3, line, res);
		d->frame = extract_pointer (5, line, res);
		d->function = extract_string (6, line, res);
		d->arguments = extract_string (7, line, res);
		d->file = extract_string (9, line, res);
		if (strlen (d->file) == 0) {
			free ((void *)d->file);
			d->file = NULL;
			d->line = -1;
		}
		else
			d->line = extract_int (10, line, res);
		dline = (void *)d;
		free (star);
		break;
	}
	case R_NEW_THREAD: {
		onewthread_t *d = malloc (sizeof (*d));
		if (d == NULL) break;	/* Memory error */
		d->id = extract_int (1, line, res);
		dline = (void *)d;
		break;
	}
default:
		/* This should never occur... */
		msg_pid_write ("\n\
#################################################################\n\
extract_line():: Case reached default statement.   ERROR!!!!!!!!!\n\
\tCode = %d\n\
#################################################################\n\
", code);
	}
	return dline;
}



PRIVATE chain_t *parse_lines (cmd_code_t cmd, const char *lines)
{
	/* Define delimiters as <NEW-LINE> */
#define DELIM "\n"
	/* Use strtok_r() if possible, strtok() otherwise */
#ifdef __USE_POSIX
	char *save;
#else
#define strtok_r(s,d,x) strtok(s,d)
#endif
	chain_t *data;
	char *line;
	char *fr;
	void *last;

	/* Request memory from stack (automatically released when leaving
	 * the function) */
	fr = alloca (strlen (lines) + 1);
	if (fr < 0 ) return NULL;	/* Error */

	/* Make a local copy of 'frames' */
	strcpy (fr, lines);

	/* Create an empty DynamicArray */
	data = chain_create ();

	/* Process the lines */
	last = NULL;
	for (line = strtok_r (fr, DELIM, &save);
		 line != NULL;
		 line = strtok_r (NULL, DELIM, &save)) {
		pcre_result res;
		int code = try_match (cmd, 0, LINE_PATTERNS, line, &res);
		if (code > E_UNKNOWN_CMD_CODE) {
			/* Something was found */
			void *d = extract_line (code, line, last, &res);
			if (d != NULL) {
				chain_enqueue (data, d);
				last = d;
			}
		}
	}
	return data;
}

/* ================================================================= */
/* ================================================================= */
/* ================================================================= */


#define OPEN(type)								\
		{ type *node = malloc (sizeof (*node));	\
		assert(node != NULL)
#define CLOSE(field)								\
		chain_enqueue (field->data, node); }
#define REP(field)								\
		node->field
#define AC(s) (s)


/* -----------------------------------------------------------------
 * Function:    PRIVATE data_t *rep = extract_reply (int code,
 *											  const char *out,
 *											  const struct re_registers *regs)
 *
 * Description: Extract the REPLY data
 *
 * Returns:     A new 'data_t' structure allocated from heap
 */
PRIVATE data_t *extract_reply (int code, const char *out,
							   const pcre_result *res)
{
	data_t *rep = f0s_tkout_data_new ();
	if (rep == NULL) return NULL; /* Memory error */

	/* Extract information assuming success */
	if (code > E_UNKNOWN_CMD_CODE)
		rep->str = extract_string (2, out, res);
	else {
		rep->str = strdup ("");
		code = -code;
	}
	rep->code = code;
	switch (code) {
	case E_UNKNOWN_CMD_CODE:
		/* Output was expected but no match was found. */
		msg_pid_write ("\n\
#################################################################\n\
REPLY_PATTERNS---Output expected but not found\n\
\tcmd_code_t = %d\n\
\tOutput =\n\
%s\n\
#################################################################\n\
",
					   code, AC (out));
		break;
	case R_KILL:
		/* No output expected */
		break;
	case R_ATTACH:
		OPEN (rattach_t);
/* 		REP (rattach_t, tp_pid) = extract_int (3, AC (out), res); */
		CLOSE (rep);
		break;
	case R_DETACH:
		OPEN (rdetach_t);
/* 		REP (rdetach_t, tp_pid) = extract_int (3, AC (out), res); */
		CLOSE (rep);
		break;
	case R_SYMBOL_FILE:
/* 		REP (rsymbolfile_t, file) = extract_string (3, AC (out), res); */
		break;
	case R_FILE:
		OPEN (rfile_t);
/* 		REP (file) = extract_string (3, AC (out), res); */
		CLOSE (rep);
		break;
	case R_RUN:
		OPEN (rrun_t);
		REP (file) = extract_string (3, AC (out), res);
		CLOSE (rep);
		break;
	case R_STEP:
	case R_NEXT:
	case R_CONTINUE:
		/* No output expected */
		break;
	case R_FINISH:
		OPEN (rfinish_t);
		REP (s_address) = extract_pointer (3, AC (out), res);
		REP (s_function) = extract_string (4, AC (out), res);
		REP (s_args) = extract_string (5, AC (out), res);
		REP (s_file) = extract_string (6, AC (out), res);
		REP (s_line) = extract_int (7, AC (out), res);
		REP (dbg_var) = extract_int (8, AC (out), res);
		REP (value) = extract_string (9, AC (out), res);
		CLOSE (rep);
		break;
	case R_CALL_VOID:
		/* No output expected */
		break;
	case R_BREAK:
		OPEN (rbreak_t);
		REP (bpid) = extract_int (3, AC (out), res);
		REP (address) = extract_pointer (4, AC (out), res);
		REP (file) = extract_string (5, AC (out), res);
		REP (line) = extract_int (6, AC (out), res);
		CLOSE (rep);
		break;
	case R_BREAK_NO_SOURCE:
		OPEN (rbreak_t);
		REP (bpid) = extract_int (3, AC (out), res);
		REP (address) = extract_pointer (4, AC (out), res);
		REP (file) = NULL;
		REP (line) = -1;
		CLOSE (rep);
		break;
	case R_DELETE:
	case R_SET_VARIABLE:
		break;
	case R_INFO_LOCALS:
		chain_destroy (rep->data);
		rep->data = parse_lines (CMD_INFO_LOCALS, rep->str);
		break;
	case R_EVALUATE:
		chain_destroy (rep->data);
		rep->data = parse_lines (CMD_EVALUATE, rep->str);
		break;
	case R_DISPLAY_EXPRESSION:
		break;
	case R_UNDISPLAY:
	case R_TTY:
		break;
	case R_LIST:
		chain_destroy (rep->data);
		rep->data = parse_lines (CMD_LIST, rep->str);
		break;
	case R_INFO_LINE:
		OPEN (rinfoline_t);
		REP (line) = extract_int (3, AC (out), res);
		REP (file) = extract_string (4, AC (out), res);
		REP (start_address) = extract_pointer (5, AC (out), res);
		REP (start_function) = extract_string (6, AC (out), res);
		REP (start_function_offset) = extract_int (8, AC (out), res);
		REP (end_address) = extract_pointer (9, AC (out), res);
		REP (end_function) = extract_string (10, AC (out), res);
		REP (end_function_offset) = extract_int (12, AC (out), res);
		CLOSE (rep);
		break;
	case R_INFO_LINE_NO_CODE:
		OPEN (rinfoline_t);
		REP (line) = extract_int (3, AC (out), res);
		REP (file) = extract_string (4, AC (out), res);
		REP (start_address) = extract_pointer (5, AC (out), res);
		REP (start_function) = extract_string (6, AC (out), res);
		REP (start_function_offset) = extract_int (8, AC (out), res);
		REP (end_address) = NULL;
		REP (end_function) = NULL;
		REP (end_function_offset) = -1;
		CLOSE (rep);
		break;
	case R_INFO_PROGRAM: {
		char *as;
		OPEN (rinfoprogram_t);
		REP (tp_pid) = extract_int (3, AC (out), res);
		REP (address) = extract_pointer (4, AC (out), res);
		REP (why) = extract_string (5, AC (out), res);
		as = extract_string (6, AC (out), res);
		if (strcmp(node->why, "breakpoint") == 0) {
			REP (bpid) = atoi (AC (as));
			free (as);
			REP (signal) = NULL;
		}
		else if (strcmp(node->why, "signal") == 0) {
			REP (bpid) = -1;
			REP (signal) = as;
		}
		else if (strcmp(node->why, "being stepped") == 0) {
			REP (bpid) = -1;
			free (as);
			REP (signal) = NULL;
		}
		else
			msg_pid_write ("\n\
#################################################################\n\
extract_reply():: Invalid value in R_INFO_PROGRAM\n\
\tString = %s\n\
#################################################################\n\
", AC (node->why));
		CLOSE (rep);
		break;
	}
	case R_INFO_STACK:
		rep->data = parse_lines (CMD_INFO_STACK, rep->str);
		break;
	case R_INFO_BREAK:
		rep->data = parse_lines (CMD_INFO_BREAK, rep->str);
		break;
	case R_INFO_DISPLAY:
		rep->data = parse_lines (CMD_INFO_DISPLAY, rep->str);
		break;
	case R_INFO_THREADS:
		rep->data = parse_lines (CMD_INFO_THREADS, rep->str);
		break;
	case R_THREAD:
		break;
	case R_SENDTO:
		OPEN (rsendto_t);
		REP (output) = rep->str;
                                /* Be carefull.  This is pointing to
								 * the same char and it's not a
								 * copy.  Do nothing in
								 * 'f0s_tkout_rep_delete()' */
		CLOSE (rep);
		break;
	default:
		/* This should never occur... */
		msg_pid_write ("\n\
#################################################################\n\
extract_reply():: Case reached default statement.   ERROR!!!!!!!!!\n\
\tCode = %d\n\
#################################################################\n\
", code);
	}
	return rep;
}



/* -----------------------------------------------------------------
 * Function:    PRIVATE data_t *oob = extract_oob (int code,
 *												 const char *out,
 *												 const struct re_registers *regs)
 *
 * Description: Extract the ALWAYS data
 *
 * Returns:     A new 'oobdata_t' structure allocated from heap
 */
PRIVATE data_t *extract_oob (int code, const char *out,
								const pcre_result *res)
{
	data_t *oob = f0s_tkout_data_new ();
	if (oob == NULL) return NULL; /* Memory error */

	/* Extract information assuming success */
	if (code > E_UNKNOWN_CMD_CODE)
		oob->str = extract_string (2, out, res);
	else {
		oob->str = strdup ("");
		code = -code;
	}
	oob->code = code;
	switch (code) {
	case E_UNKNOWN_CMD_CODE:
		/* No match was found.  Try next set of patterns. */
		break;
	case O_DISPLAY:
		oob->data = parse_lines (CMD_DISPLAY, oob->str);
		break;
	case O_BPHIT:
		OPEN (obphit_t);
		REP (bpid) = extract_int (3, out, res);
		REP (function) = extract_string (4, out, res);
		REP (args) = extract_string (5, out, res);
		REP (file) = extract_string (7, out, res);
		REP (line) = extract_int (8, out, res);
		CLOSE (oob);
		break;
	case O_SIGNAL:
		OPEN (osignal_t);
		REP (name) = extract_string (3, out, res);
		CLOSE (oob);
		break;
	case O_NEW_THREAD:
		oob->data = parse_lines (CMD_NEW_THREAD, oob->str);
		break;
	case O_SWITCH_TO_THREAD:
		OPEN (othread_t);
		REP (pid) = extract_int (3, out, res);
		CLOSE (oob);
		break;
	default:
		/* This should never occur... */
		msg_pid_write ("\n\
#################################################################\n\
extract_oob():: Case reached default statement.   ERROR!!!!!!!!!\n\
\tCode = %d\n\
#################################################################\n\
", code);
	}
	return oob;
}


/* -----------------------------------------------------------------
 * Function:    PRIVATE data_t *pos = extract_position (int code,
 *												 const char *out,
 *												 const struct re_registers *regs)
 *
 * Description: Extract the POSITION data
 *
 * Returns:     A new 'data_t' structure allocated from heap
 */
PRIVATE data_t *extract_position (int code, const char *out,
								  const pcre_result *res)
{
	data_t *pos = f0s_tkout_data_new ();
	if (pos == NULL) return NULL; /* Memory error */

	/* Extract information assuming success */
	if (code > E_UNKNOWN_CMD_CODE)
		pos->str = extract_string (2, out, res);
	else {
		pos->str = strdup ("");
		code = -code;
	}
	pos->code = code;
	switch (code) {
	case E_UNKNOWN_CMD_CODE:
		/* No match was found.  Try next set of patterns. */
		break;
	case P_POSITION:
		OPEN (pposition_t);
		REP (file) = extract_string (3, out, res);
		REP (line) = extract_int (4, out, res);
		REP (pos) = extract_int (5, out, res);
		REP (where) = extract_string (6, out, res);
		REP (frame) = extract_pointer (7, out, res);
		CLOSE (pos);
		break;
	default:
		/* This should never occur... */
		msg_pid_write ("\n\
#################################################################\n\
extract_position():: Case reached default statement.   ERROR!!!!!!!!!\n\
\tCode = %d\n\
#################################################################\n\
", code);
	}
	return pos;
}



/* -----------------------------------------------------------------
 * Function:    PRIVATE data_t *err = extract_error (int code,
 *												 const char *out,
 *												 const struct re_registers *regs)
 *
 * Description: Extract the ERROR data
 *
 * Returns:     A new 'data_t' structure allocated from heap
 */
PRIVATE data_t *extract_error (int code, const char *out,
							   const pcre_result *res)
{
	data_t *err = f0s_tkout_data_new ();
	if (err == NULL) return NULL; /* Memory error */

	/* Extract information assuming success */
	if (code > E_UNKNOWN_CMD_CODE)
		err->str = extract_string (2, out, res);
	else {
		err->str = strdup ("");
		code = -code;
	}
	err->code = code;
	switch (code) {
	case E_UNKNOWN_CMD_CODE:
		/* No match was found.  Try next set of patterns. */
		break;
	case E_NOT_RUN:
	case E_NO_FRAME_SELECTED:
	case E_NO_PROCESS:
	case E_NOT_PERMITTED:
	case E_NO_SYMBOL:
	case E_BOTTOM:
	case E_TOP:
	case E_INVALID_EXPRESSION:
	case E_NO_SYMBOLS:
	case E_NO_SYMBOL_TABLE:
	case E_NO_DEF_SOURCE_FILE:
	case E_INVALID_BPID:
	case E_INVALID_LINE_NUMBER:
	case E_FUNCTION_NOT_DEFINED:
	case E_LINE_OUT_OF_RANGE:
	case E_NO_BOUNDS:
	case E_INVALID_SYMBOL:
	case E_INVALID_DISPLAY_NUMBER:
	case E_NO_DISPLAY_EXPRESSION:
	case E_INVALID_ARITHMETIC_OPERATION:
	case E_PARSE_ERROR:
	case E_TOO_FEW_ARGS:
	case E_NO_STACK:
	case E_NO_BREAK_OR_WATCH:
	case E_NO_LINE_NUM_INFO:
	case E_JUNK_AT_END_OF_LINE:
	case E_INVALID_THREAD_ID:
	case E_NO_THREAD_ID:
	case E_NO_THREAD_COMMAND:
		break;
	default:
		/* This should never occur... */
		msg_pid_write ("\n\
#################################################################\n\
extract_error():: Case reached default statement.   ERROR!!!!!!!!!\n\
\tCode = %d\n\
#################################################################\n\
", code);
	}
	return err;
}

#undef OPEN
#undef CLOSE
#undef REP
#undef AC


/* #################################################################
 *		PUBLIC functions
 * ################################################################# */

/* -----------------------------------------------------------------
 * Function:    int status = f0s_init_parsing (PatRec *patterns[])
 *
 * Description: Initialize the data structures for parsing the
 *				debugger's output.
 *
 * Returns:     The status of the operation
 */
int f0s_init_parsing (PatRec *patterns[])
{
        /*
	int i;
	
	for (i = 0;  patterns[i] != NULL;  i++)
		if (initialize_patterns (patterns[i]) < 0)
			return -1;
	*/
        if (f0s_load_patterns() >0) return -1;
  
	return 0;
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_parse_output (int tid,
 *												 cmd_code_t cmdcode,
 *											     const char *out)
 *
 * Description: Parse the string 'out' assuming its the output of
 *				command 'cmd_code'
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_parse_output (int tid, cmd_code_t cmdcode, const char *out)
{
	int code;
	pcre_result res;
	tkout_t *tk;
	int status = 0;

	/* Compile regular expressions if necessary */
	if ((! intialized)  &&  (f0s_init_parsing (patterns) == -1))
		return f0s_tkout_new (E_RTS_INIT_REGEX);
	else
	        intialized=1;

	/* Create a new 'Output Token' */
	tk = f0s_tkout_new (E_RTS_OK);

	/* Set tid and the output string in token */
	tk->tid = tid;
	tk->str = out;

	/* -------------------------------------------------- */
	/* Extract information from the "reply" patterns */
	code = try_match (cmdcode, 0, REPLY_PATTERNS,
					  out, &res);
	if (code != E_UNKNOWN_CMD_CODE) {
		/* Some output was found.  Probably the command succeeded
		 * (this is not for sure!) */
		tk->rep = extract_reply (code, out, &res);
		status = 1;
	}

	/* -------------------------------------------------- */
	/* Extract information from the "oob" patterns.  All the
	 * patterns must be tested, and not just unitil one matches! */
	for (code = -1; ; ) {
		data_t *a;
		code = try_match (cmdcode, code, ALWAYS_PATTERNS,
						  out, &res);
		if (code == E_UNKNOWN_CMD_CODE)
			break;
		/* Some "oob" output was found. */
		a = extract_oob (code, out, &res);
		chain_enqueue (tk->oob, a);
		status = 1;
	}
	
	/* -------------------------------------------------- */
	/* Extract information from the "position" pattern */
	code = try_match (cmdcode, 0, POSITION_PATTERNS,
					  out, &res);
	if (code != E_UNKNOWN_CMD_CODE) {
		/* Some positioning output was found. */
		tk->pos = extract_position (code, out, &res);
		status = 1;
	}
	
	/* -------------------------------------------------- */	
	/* Extract information from the "error" patterns */
	code = try_match (cmdcode, 0, ERROR_PATTERNS,
					  out, &res);
	if (code != E_UNKNOWN_CMD_CODE) {
		/* Some output was found.  Probably the command failed */
		tk->err = extract_error (code, out, &res);
		status = 1;
	}

	if (status == 0)
		/* Unable to parse the output */
		tk->status = E_RTS_UNKNOWN_OUTPUT;
	return tk;
}



