#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <pcre.h>
#include <limits.h>
#include <assert.h>
#include "glob.h"
#include "chain.h"
#include "f0s_pcre_file.h"

#define SKIP_SPACES_AND_TABS while (*ptr==' ' || *ptr=='\t') {ptr++;} //skip spaces and tabs
#define SKIP_TO_NEXT_SPACE_OR_TAB while (*ptr!=' ' && *ptr!='\t') {ptr++;} //skip to next space or tab


#define MAX(a,b) ((a>b?a:b))

PatRec *patterns[NUM_PATTERNS];

int max_patterns = 0;

typedef struct { char *str; int code; } code_table_t;

code_table_t gdb_cmd_table[] =
{
  {"CMD_UNUSED", CMD_UNUSED},
  {"CMD_NO_CMD", CMD_NO_CMD},
  {"CMD_TERMINATE", CMD_TERMINATE},
  {"CMD_TIDS", CMD_TIDS},
  {"CMD_KILL", CMD_KILL},
  {"CMD_ATTACH", CMD_ATTACH},
  {"CMD_DETACH", CMD_DETACH},
  {"CMD_SYMBOL_FILE", CMD_SYMBOL_FILE},
  {"CMD_FILE", CMD_FILE},
  {"CMD_RUN", CMD_RUN},
  {"CMD_STEP", CMD_STEP},
  {"CMD_NEXT", CMD_NEXT},
  {"CMD_CONTINUE", CMD_CONTINUE},
  {"CMD_FINISH", CMD_FINISH},
  {"CMD_SIGNAL", CMD_SIGNAL},
  {"CMD_BREAK", CMD_BREAK},
  {"CMD_DELETE", CMD_DELETE},
  {"CMD_SET_VARIABLE", CMD_SET_VARIABLE},
  {"CMD_EVALUATE", CMD_EVALUATE},
  {"CMD_INFO_LOCALS", CMD_INFO_LOCALS},
  {"CMD_INFO_ARGS", CMD_INFO_ARGS},
  {"CMD_DISPLAY", CMD_DISPLAY},
  {"CMD_UNDISPLAY", CMD_UNDISPLAY},
  {"CMD_CALL", CMD_CALL},
  {"CMD_LIST", CMD_LIST},
  {"CMD_TTY", CMD_TTY},
  {"CMD_INFO_LINE", CMD_INFO_LINE},
  {"CMD_INFO_PROGRAM", CMD_INFO_PROGRAM},
  {"CMD_INFO_STACK", CMD_INFO_STACK},
  {"CMD_UP", CMD_UP},
  {"CMD_DOWN", CMD_DOWN},
  {"CMD_FRAME", CMD_FRAME},
  {"CMD_INFO_BREAK", CMD_INFO_BREAK},
  {"CMD_INFO_DISPLAY", CMD_INFO_DISPLAY},
  {"CMD_THREAD", CMD_THREAD},
  {"CMD_INFO_THREADS", CMD_INFO_THREADS},
  {"CMD_SENDTO", CMD_SENDTO},
  {"CMD_NEW_THREAD", CMD_NEW_THREAD},
  {NULL,0}
};

code_table_t reply_table[] =
{
   /* The error codes are included in the interest of thouroughness, 
    they will never be encountered in the file */
  {"FIDDLE_OK", FIDDLE_OK},
  {"E_RTS_OK", E_RTS_OK},
  {"E_RTS_SEND_RECEIVE", E_RTS_SEND_RECEIVE},
  {"E_RTS_INVALID_ND_PID", E_RTS_INVALID_ND_PID},
  {"E_RTS_NO_CLD_AVAILABLE", E_RTS_NO_CLD_AVAILABLE},
  {"E_RTS_COMMAND_GENERATION", E_RTS_COMMAND_GENERATION},
  {"E_RTS_INVALID_TID", E_RTS_INVALID_TID},
  {"E_RTS_STAT_FILE", E_RTS_STAT_FILE},
  {"E_RTS_FILE_PERM", E_RTS_FILE_PERM},
  {"E_RTS_STAT_TTY", E_RTS_STAT_TTY},
  {"E_RTS_TTY_PERM", E_RTS_TTY_PERM},
  {"E_RTS_ONE_LINE_FUNCTION", E_RTS_ONE_LINE_FUNCTION},
  {"E_RTS_NO_COMMAND", E_RTS_NO_COMMAND},
  {"E_RTS_ASTR_CREATE", E_RTS_ASTR_CREATE},
  {"E_RTS_INIT_REGEX", E_RTS_INIT_REGEX},
  {"E_RTS_INVALID_LINE_NUMBER", E_RTS_INVALID_LINE_NUMBER},
  {"E_RTS_INVALID_ARGUMENT", E_RTS_INVALID_ARGUMENT},
  {"E_RTS_INVALID_OPTION", E_RTS_INVALID_OPTION},
  {"E_RTS_MEMORY_ERROR", E_RTS_MEMORY_ERROR},
  {"E_RTS_INVALID_SIGNUM", E_RTS_INVALID_SIGNUM},
  {"E_RTS_SIGNAL", E_RTS_SIGNAL},
  {"E_RTS_INVALID_REQID", E_RTS_INVALID_REQID},
  {"E_RTS_CANNOT_START_DAEMON", E_RTS_CANNOT_START_DAEMON},
  {"E_RTS_UNKNOWN_OUTPUT", E_RTS_UNKNOWN_OUTPUT},
  {"E_UNKNOWN_CMD_CODE", E_UNKNOWN_CMD_CODE},
  {"E_TOP", E_TOP},
  {"E_NOT_RUN", E_NOT_RUN},
  {"E_NO_FRAME_SELECTED", E_NO_FRAME_SELECTED},
  {"E_NO_PROCESS", E_NO_PROCESS},
  {"E_NOT_PERMITTED", E_NOT_PERMITTED},
  {"E_NO_SYMBOL", E_NO_SYMBOL},
  {"E_BOTTOM", E_BOTTOM},
  {"E_INVALID_EXPRESSION", E_INVALID_EXPRESSION},
  {"E_NO_SYMBOLS", E_NO_SYMBOLS},
  {"E_NO_SYMBOL_TABLE", E_NO_SYMBOL_TABLE},
  {"E_NO_DEF_SOURCE_FILE", E_NO_DEF_SOURCE_FILE},
  {"E_INVALID_BPID", E_INVALID_BPID},
  {"E_INVALID_LINE_NUMBER", E_INVALID_LINE_NUMBER},
  {"E_FUNCTION_NOT_DEFINED", E_FUNCTION_NOT_DEFINED},
  {"E_LINE_OUT_OF_RANGE", E_LINE_OUT_OF_RANGE},
  {"E_NO_BOUNDS", E_NO_BOUNDS},
  {"E_INVALID_SYMBOL", E_INVALID_SYMBOL},
  {"E_INVALID_DISPLAY_NUMBER", E_INVALID_DISPLAY_NUMBER},
  {"E_NO_DISPLAY_EXPRESSION", E_NO_DISPLAY_EXPRESSION},
  {"E_INVALID_ARITHMETIC_OPERATION", E_INVALID_ARITHMETIC_OPERATION},
  {"E_PARSE_ERROR", E_PARSE_ERROR},
  {"E_TOO_FEW_ARGS", E_TOO_FEW_ARGS},
  {"E_NO_STACK", E_NO_STACK},
  {"E_NO_BREAK_OR_WATCH", E_NO_BREAK_OR_WATCH},
  {"E_NO_LINE_NUM_INFO", E_NO_LINE_NUM_INFO},
  {"E_JUNK_AT_END_OF_LINE", E_JUNK_AT_END_OF_LINE},
  {"E_INVALID_THREAD_ID", E_INVALID_THREAD_ID},
  {"E_NO_THREAD_ID", E_NO_THREAD_ID},
  {"E_NO_THREAD_COMMAND", E_NO_THREAD_COMMAND},
  {"E_SIGSEGV", E_SIGSEGV},
  {"E_LAST", E_LAST},
   
   /******************** Reply Codes *******************************/
  {"R_FIRST", R_FIRST},
  {"R_ATTACH", R_ATTACH},
  {"R_BREAK", R_BREAK},
  {"R_BREAK_NO_SOURCE", R_BREAK_NO_SOURCE},
  {"R_CALL_VOID", R_CALL_VOID},
  {"R_CONTINUE", R_CONTINUE},
  {"R_DELETE", R_DELETE},
  {"R_DETACH", R_DETACH},
  {"R_EVALUATE", R_EVALUATE},
  {"R_INFO_LOCALS", R_INFO_LOCALS},
  {"R_FILE", R_FILE},
  {"R_FINISH", R_FINISH},
  {"R_INFO_BREAK", R_INFO_BREAK},
  {"R_INFO_BREAK_WATCH_DATA", R_INFO_BREAK_WATCH_DATA},
  {"R_INFO_DISPLAY", R_INFO_DISPLAY},
  {"R_INFO_LINE", R_INFO_LINE},
  {"R_INFO_LINE_NO_CODE", R_INFO_LINE_NO_CODE},
  {"R_INFO_PROGRAM", R_INFO_PROGRAM},
  {"R_INFO_STACK", R_INFO_STACK},
  {"R_INFO_THREADS", R_INFO_THREADS},
  {"R_INFO_WATCHPOINT", R_INFO_WATCHPOINT},
  {"R_KILL", R_KILL},
  {"R_LIST", R_LIST},
  {"R_NEXT", R_NEXT},
  {"R_RUN", R_RUN},
  {"R_SENDTO", R_SENDTO},
  {"R_SET_VARIABLE", R_SET_VARIABLE},
  {"R_STEP", R_STEP},
  {"R_SYMBOL_FILE", R_SYMBOL_FILE},
  {"R_THREAD", R_THREAD},
  {"R_TIDS", R_TIDS},
  {"R_ADD_DEBUGGER", R_ADD_DEBUGGER},
  {"R_TTY", R_TTY},
  {"R_UNDISPLAY", R_UNDISPLAY},
  {"R_NEW_THREAD", R_NEW_THREAD},
  {"R_CALL", R_CALL},
  {"R_EVALUATE_ONE_LINE", R_EVALUATE_ONE_LINE},
  {"R_EVALUATE_MANY_START", R_EVALUATE_MANY_START},
  {"R_EVALUATE_MANY_MIDDLE", R_EVALUATE_MANY_MIDDLE},
  {"R_EVALUATE_MANY_END", R_EVALUATE_MANY_END},
  {"R_INFO_LOCALS_ONE_LINE", R_INFO_LOCALS_ONE_LINE},
  {"R_INFO_LOCALS_MANY_START", R_INFO_LOCALS_MANY_START},
  {"R_INFO_LOCALS_MANY_MIDDLE", R_INFO_LOCALS_MANY_MIDDLE},
  {"R_INFO_LOCALS_MANY_END", R_INFO_LOCALS_MANY_END},
  {"R_DISPLAY_EXPRESSION", R_DISPLAY_EXPRESSION},
  {"R_DISPLAY_ONE_LINE", R_DISPLAY_ONE_LINE},
  {"R_DISPLAY_MANY_START", R_DISPLAY_MANY_START},
  {"R_DISPLAY_MANY_MIDDLE", R_DISPLAY_MANY_MIDDLE},
  {"R_DISPLAY_MANY_END", R_DISPLAY_MANY_END},
  {"R_CLIENTS", R_CLIENTS},
  {"R_REGISTER_CLIENT", R_REGISTER_CLIENT},
  {"R_LAST", R_LAST},

   /******************* Position Codes *************************/
  {"P_FIRST", P_FIRST},
  {"P_POSITION", P_POSITION},
  {"P_LAST", P_LAST},

   /******************* Out-of-Band Codes **********************/
  {"O_FIRST", O_FIRST},
  {"O_DISPLAY", O_DISPLAY},
  {"O_BPHIT", O_BPHIT},
  {"O_SIGNAL", O_SIGNAL},
  {"O_NEW_THREAD", O_NEW_THREAD},
  {"O_SWITCH_TO_THREAD", O_SWITCH_TO_THREAD},
  {"O_LAST", O_LAST},
  {NULL,0}
};

/*
 * Translate a string to the apropriate Command code
 * 
 * IN: the string with the code description (eg.: CMD_KILL)
 * IN: Where to search, in CMD_TABLE or REPLY_TABLE
 * 
 * Returns: the code, or -1 on error
*/
PRIVATE int translate_code(char *code_string,int where)
{
  int i;


  switch (where) {
  case CMD_TABLE: /* we were told to search in the command table */
    for (i=0;gdb_cmd_table[i].str!=NULL;i++) {
      if (!strcmp(code_string,gdb_cmd_table[i].str))
	return gdb_cmd_table[i].code;
    }
    break;
  case REPLY_TABLE: /* we were told to search in the reply table */
    for (i=0;reply_table[i].str!=NULL;i++) {
      if (!strcmp(code_string,reply_table[i].str))
	return reply_table[i].code;
    }
    break;
  default: /* if not in the CMD_TABLE or in the REPLY_TABLE, return error */
    return -1;
  };
  return -1; /* Can't find that code */
}

/*
 * Open the file and read it all to a memory Buffer
 *
 * IN: The file name
 *
 * Returns: a Pointer to a char buffer with the file contents
 *
 * Note: it's the responsability of the caller to free the memory 
 *       ocuppied by buffer returned in this funcion
*/
PRIVATE char *open_pattern_file(char *filename)
{
  FILE *patfile;
  int filesize,count;
  char *ret;


  /* Open the File */
  patfile=fopen(filename,"r");
  if (patfile==NULL) {
    msg_pid_fatal("Could not open file: %s\n",filename);
  }


  /* Find out the file size */
  if ( fseek(patfile,0,SEEK_END) != 0 ) {
    msg_pid_fatal("Could not determine file size: %s\n",filename);
  }
  filesize=ftell(patfile);
  if (filesize<0) {
    msg_pid_fatal("Could not determine file size: %s\n",filename);
  }
  rewind(patfile);



  ret=(char *)malloc(filesize+1);  /* Allocate Memory for the file */

  clearerr(patfile); /* Clear the FILE struct's error flag */

  count=fread(ret,filesize,1,patfile); /* Read the file */

  if (count<filesize) { /* Check to see if we really read the whole file*/
    if ( feof(patfile) ) { /* EOF reached prematurely? */
      msg_pid_fatal("Unexpected EOF while loading patterns.\n");
    } else
      if ( ferror(patfile) ) { /* error flag is set? */
	/* MISSING: Error handling */
	printf("f0s_pcre_file.c:282");
	exit(-1);
      }
  }

  /* Cleanup - close the file */
  fclose(patfile);

  /* return the pointer to the buffer */
  ret[filesize]='\0';
  return ret;
}







/*
 * Insert a new pattern in the temp pattern structs, including compiling it.
 *
 * IN: Pattern type, cmdcode, replycode and pattern text
 *
 * Returns: 0 on success, positive number on error
*/
PRIVATE int insert_pattern(int type, int cmdcode, int replycode, char *pattern, chain_t **temp)
{
  PatRec *new;
  const char *pcre_error;
  int err_offset;

  /* Check if type is in correct range of values: [0..4]*/
  if (type<0 || type>4) {
    msg_pid_fatal("Pattern type not in expected range: %d (should be in [0..4])\n",type);
  }

  new=(PatRec *)malloc(sizeof(PatRec));

  new->cmdcode=cmdcode;
  new->outcode=replycode;
  if (!strcmp(pattern,"NULL"))
    new->pat=NULL;
  else {
    new->pat=pcre_compile(pattern, //pattern to compile
			  0, //default options
			  &pcre_error, //string to write eventual errors to
			  &err_offset, //offset of the error-causing character  in the pattern
			  NULL); //default character tables

    /* Check for errors in pattern compilation*/
    if (new->pat==NULL) {
      msg_pid_fatal("Error compiling Pattern:\n%s\nAt character %d. PCRE has this to say:\n%s\n",pattern,err_offset,pcre_error);
    }

    new->extra=pcre_study(new->pat, //the compiled pattern
			  0, //default options
			  &pcre_error); //string to write eventual errors to
    
    /* check for errors in pattern study */
    if ( pcre_error != NULL ) {
      msg_pid_fatal("Error while studying pattern:\n%s\nPCRE has this to say:\n%s\n",pattern,pcre_error);
    }
  }

  /******* Pattern ready, now to insert in the temp structure *****/


  if (chain_enqueue(temp[type],(void *)new) == NULL) {
    msg_pid_fatal("Could not insert new pattern into temporary structure, chain_enqueue() returned an error.\n");
  }
  

  return 0;
}









/*
 * Move the patterns from the temp structure to the final structure.
 *
 * Returns: 0 on success, positive number on error
 *
*/
PRIVATE int move_to_struct(chain_t **temp)
{
  int p,len,i;
  chain_node_t *node;

  for (p=0;p<NUM_PATTERNS;p++) {
    len=chain_length(temp[p]);
    patterns[p]=(PatRec *)malloc(sizeof(PatRec)*len);
    node=chain_first(temp[p]);
    for (i=0;i<len;i++) {
      memcpy(&patterns[p][i],(PatRec *)chain_node_data(node),sizeof(PatRec));
      node=chain_next(temp[p],node);
    }
  }

  return 0;
}











PUBLIC int f0s_load_patterns()
{
  char filename[PATH_MAX],*buf,buf2[PATH_MAX],*ptr,*line_b,*line_e,*eof,*pat_pattern,*walker;
  int pat_type,pat_cmdcode,pat_replycode,i,count;
  chain_t *temp[NUM_PATTERNS];


  for (i=0;i<NUM_PATTERNS;i++)
    temp[i]=chain_create(); //create temporary holding structure

  /* MISSING: Check if chain_create succeeded (how?) */

  //buf=(char *)malloc(PATH_MAX); //temporarily allocate memory 

  /* determine file name: if no env var of name PATTERN_FILE exists,
   * use default, else use PATTERN_FILE. */
  buf=getenv("PATTERN_FILE");

  if (buf==NULL) {
    strcpy (filename,PATTERN_FILE);
  } else {
    strcpy(filename,buf);
  }
  //  strcpy(filename,PATTERN_FILE);

  //free(buf);  //free the allocated memory

  //open the file (call open_pattern_file)
  buf=open_pattern_file(filename);

  //separate the lines and extract the information
  ptr=buf;
  eof=buf+strlen(buf);
  line_b=ptr;
  line_e=line_b;

  while (line_b < eof) {
    while (*line_e!='\n' && line_e != '\0') {line_e++;} //find the end of this line, watch for eof

    if (line_e=='\0') { //reached EOF
      line_b=line_e;
      continue;
    }

    if (*line_b=='#') { //skip comments
      line_b=++line_e;
      continue; 
    }

    if (*line_b=='\n') { //skip empty lines
      line_b=++line_e;
      continue;
    }
    //MISSING: more complex cases of empty lines, such as all spaces, tabs or a combination of these.


    ptr=line_b;    

    //read the pattern's type
    SKIP_SPACES_AND_TABS;
    sscanf(ptr,"%d",&pat_type);
    SKIP_TO_NEXT_SPACE_OR_TAB;

    //read the pattern's command code
    SKIP_SPACES_AND_TABS;
    sscanf(ptr,"%s",buf2);
    pat_cmdcode=translate_code(buf2,CMD_TABLE); //translate the string to a code the program knows.
    if (pat_cmdcode==-1) {
      /*MISSING: Error Handling*/ 
      printf("Unknown Command Code: %s\n",buf2);
      assert(pat_cmdcode!=-1);
    }
    SKIP_TO_NEXT_SPACE_OR_TAB;

    //read the pattern's reply code
    SKIP_SPACES_AND_TABS;
    sscanf(ptr,"%s",buf2);
    pat_replycode=translate_code(buf2,REPLY_TABLE); //translate the string to a code the program knows.

    if (pat_replycode==-1) {
      /*MISSING: Error Handling*/ 
      printf("Unknown Reply Code: %s\n",buf2);
      assert(pat_replycode!=-1);
    }
    SKIP_TO_NEXT_SPACE_OR_TAB;

    //read the pattern's pattern string
    SKIP_SPACES_AND_TABS;
    pat_pattern=(char *)malloc((line_e-ptr)+1);
    strncpy(pat_pattern,ptr,line_e-ptr);
    pat_pattern[line_e-ptr]='\0';

    //insert all this into the structure
    if (insert_pattern(pat_type,pat_cmdcode,pat_replycode,pat_pattern,temp) >0) {
      //MISSING: Error handling (msg_pid_fatal)
      printf("f0s_pcre_file.c:491");
      exit(-1);
    }

    //check to see if this is the largest number of pattern captures yet
    count=0;
    for (walker=pat_pattern;*walker!='\0';walker++) {
      if (*walker=='\\') walker++;
      else if (*walker=='(' && *(walker+1)!='?') count++;
    }
    max_patterns=MAX(max_patterns,count);

    //free string's buffer memory
    free(pat_pattern);

    line_b=++line_e;
  }
  

  for (i=0;i<NUM_PATTERNS;i++) {
    PatRec *novo=(PatRec *)malloc(sizeof(*novo));
    novo->cmdcode=CMD_EOLIST;
    chain_enqueue(temp[i],(void *)novo);
  }


  if (move_to_struct(temp) >0) { //move data from the temp struct to the final struct
    /* MISSING: Error Handling */
    printf("f0s_pcre_file.c:519");
    exit(-1);
  }



  for (i=0;i<NUM_PATTERNS;i++)
    chain_destroy(temp[i]);//destroy the chain


  free(buf);
}
