/* ******************************************************************
 *
 *  $Id: Fiddle_j.java,v 1.1 2002/08/31 15:23:00 rja Exp $
 *
 *  Copyright (c) 2000 by Dep.Informatica of the FCT/UNL
 *  Time-stamp: <02/08/27 18:33:03 rja>
 *
 *  Ricardo Anastacio [jml] <rja@asc.di.fct.unl.pt>
 *
 *  Parallel and Distributed Processing Group
 *  Departamento de Informatica
 *  Faculdade de Ciencias e Tecnologia
 *  Universidade Nova de Lisboa
 *
 * -----------------------------------------------------------------
 *
 *  Description:
 *
 * Class Fiddle_j will represent from the JAVA side the fiddle 
 * engine.
 *
 * -----------------------------------------------------------------
 */

package unl.fct.di.fiddle_j;


public class Fiddle_j {

/**
 * Constructs a Fiddle_j object and initializes fiddle.
 * Throws FiddleNotInitializedException if the fiddle
 * initialization process didn't succeed.
 */
	 public Fiddle_j() throws FiddleNotInitializedException {
		super();

		//Start the fiddle machine
//		if(initialize("localhost", "44444", 1) < 0)
		if(initialize(null, null, 1) < 0)
			 throw new FiddleNotInitializedException("Fiddle machine not initialized! f1m_initialize returned a negative number");
	 }

	 public Fiddle_j(String host, String service, boolean startServer, boolean keepServer) throws FiddleNotInitializedException {
		  super();

		  //Build flags
		  int flags = 0;
		  if(startServer) flags += 1;
		  if(keepServer) flags += 2;

		  //Start the fiddle machine
		  if(initialize(host, service, flags) < 0)
			 throw new FiddleNotInitializedException("Fiddle machine not initialized! f2m_initialize returned a negative number");
	 }

	 protected void finalize() {
		  die(0);
	 }

	 static {
		  System.loadLibrary("fiddle_j");
	 }
	 
	 
	 /* Process control native methods */

/**
 * Initializes the fiddle machine
 */
	 private native int initialize(String host, String service, int flags);

/**
 * Terminates the fiddle machine
 */
	 private native void die(int flag_halt);




/**
 * Terminates this client and the fiddle machine.
 */
	 public void terminate() {
		  die(1);
	 }


/*  ------------ Exception throwing methods -------------
 * All the next methods return the following exceptions:
 * Throws NullPointerException if the fiddle machine 
 * returned a null tkout.
 * Throws FiddleException if an error occured while
 * processing the request.
 * Throws DebuggerException if the debugger wasn't able
 * to sucessfully execute the request.
 * Throws Exception if, by any chance, none of the
 * above exceptions could be thrown.
 * -----------------------------------------------------
 */





// ---- Management Methods --->>> 

/**
 * Lists the several Task IDs in Fiddle and its state.
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j tids() throws NullPointerException, FiddleException;

/**
 * For private use of methods clients_with_name(...) and
 * clients_in_machine(...).
 */
	 private native Tkout_j clients(String option) throws NullPointerException, FiddleException, ServiceNotAvailableException;

/**
 * Returns all the clients registered to this fiddle machine
 * with the specified name.
 * It throws ServiceNotAvailableException if this package wasn't
 * compiled with support of layer f2m.
 * (+) name, the clients name;
 * (-) the JAVA image of the Tkout structure.
 */
	 public Tkout_j clients_with_name(String name) throws NullPointerException, FiddleException, ServiceNotAvailableException {
		  return clients("-n" + name);
	 }

/**
 * Returns all the clients registered to this fiddle machine
 * with the specified name.
 * It throws ServiceNotAvailableException if this package wasn't
 * compiled with support of layer f2m.
 * (+) machine, the machine's name or IP string address with the 
 *     registered clients;
 * (-) the JAVA image of the Tkout structure.
 */
	 public Tkout_j clients_in_machine(String machine) throws NullPointerException, FiddleException, ServiceNotAvailableException {
		  return clients("-m" + machine);
	 }







// ---- Process Control Methods --->>> 

/**
 * Attaches this Fiddle_j to an already running process
 * (+) machine, the name of the machine to connect to;
 * (+) tp_pid, the pid of the target process;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j attach(String machine, int tp_pid) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Detaches the back-end debugger from the process. The process
 * is left to run uncontrolled as it was before attachment and
 * the back-end bebugger is released.
 * (+) tid, the program's Task ID;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j detach(int tid) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Kills the target process associated with the symbolic Task ID.
 * (+) tid, the program's Task ID;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j kill(int tid) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Loads the symbols table for the given Task ID from file.
 * (+) tid, the program's Task ID;
 * (+) machine, the machine's name where the file is located;
 * (+) file, the file with the symbols table;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j symbol_file(int tid, String machine, String file) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Loads the executable file and reads its symbol table 
 * into memory.
 * (+) machine, the name of the machine to connect to;
 * (+) file, the name of the executable to load;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j file(String machine, String file) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Starts the execution of a previosly loaded program.
 * (+) tid, the program's Task ID;
 * (+) args, a list of arguments, separated by a space to
 *           pass to the debugger. This list can be an 
 *           empty string of even point to null, if no
 *           extra arguments are needed to be passed;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j run(int tid, String args) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Continues the execution of the given Task ID until
 * the next instruction is reach, stepping into functions
 * if it's the case.
 * (+) tid, the program's Task ID;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j step(int tid) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Continues the execution of the given Task ID until
 * the next instruction is reach.
 * (+) tid, the program's Task ID;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j next(int tid) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Continues the execution of a job previously stopped
 * by means of a signal or a breakpoint being hit.
 * (+) tid, the program's Task ID;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j continueExecution(int tid) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Continues the executation until the current stack
 * frame returns.
 * (+) tid, the program's Task ID;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j finish(int tid) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Sends a signal to the process identified by tid.
 * If the process is stopped, it will receive the signal when 
 * the execution is resumed, otherwise the process execution 
 * is stopped. All the signals except SIGINT will be passed to 
 * the program. The SIGINT signal will just interrupt the 
 * program execution without further cosnequences.
 * (+) tid, the program's Task ID;
 * (+) signum, the signal number code to send;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j signal(int tid, int signum) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Sets a breakpoint in line or function of process tid. This two arguments are
 * exclusive. So if line is set, function should be an empty string or NULL.
 * Otherwise, function should contain a valid function name and line should be -1.
 * File can be omited by passing a NULL argument or an empty string. If so the 
 * current source file will be used.
 * (+) tid, the program's Task ID;
 * (+) file, the name of the source file;
 * (+) line, the line number on which to insert the breakpoint;
 * (+) function, the name of the function to stop;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j setBreakpoint(int tid, String file, int line, String function) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Deletes a breakpoint.
 * (+) tid, the Task ID;
 * (+) bpid, the breakpoint ID.
 * (-) The JAVA image of the Tkout structure
 */
	 public native Tkout_j delete(int tid, int bpid) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Sets the value of the given variable to the result of the 
 * evaluation of the given expression in the current context 
 * of the Task ID.
 * (+) tid, the program's Task ID;
 * (+) variable, the name of the variable to alter;
 * (+) expression, the expression to evaluate and assign to the variable;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j set_variable(int tid, String variable, String expression) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Evaluates a function call.
 * (+) tid, the program's Task ID;
 * (+) expression, the function name and its arguments;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j call(int tid, String expression) throws NullPointerException, FiddleException, DebuggerException;







// ---- Process Inspection Methods --->>> 

/**
 * Evaluates the given expression within the current context
 * of the Task ID.
 * (+) tid, the program's Task ID;
 * (+) expression, the string expression to evaluate;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j evaluate(int tid, String expression) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Evaluates the given expression int the current context of 
 * the Task ID every time the process stops running.
 * (+) tid, the program's Task ID;
 * (+) expression, expression to be evaluated;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j display(int tid, String expression) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Removes the expression associated with the given display ID
 * from the list of expressions to be evaluated every time the 
 * process stops running.
 * (+) tid, the program's Task ID;
 * (+) dispid, the display ID to be removed;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j undisplay(int tid, int dispid) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Lists n lines of a source file of tid, starting at a 
 * line or the beggining of a function (exclusive).
 * (+) tid, the program's Task ID;
 * (+) file, the source file (null == current source file);
 * (+) start_line, the line to start listing;
 * (+) function, the name of the function to list;
 * (+) n_lines, the number of lines to list after the first;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j list(int tid, String file, int start_line, String function, int n_lines) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Gets information about a source line of the
 * process identifid by tid.	 
 * (+) tid, the program's Task ID;
 * (+) file, the source filename. This can be null
 *     indicating to use the current file;
 * (+) line, the line number to get info about
       or -1 if function != null;
 * (+) function, the function name
       or null if line != -1;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j info_line(int tid, String file, int line, String function) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Gets information about the program tid.
 * (+) tid, the program's Task ID;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j info_program(int tid) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Gets information about the stack of tid (backtrace).
 * (+) tid, the program's Task ID;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j info_stack(int tid) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Changes the current stack frame of tid one layer up.
 * (+) tid, the program's Task ID;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j up(int tid) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Changes the current stack frame of tid one layer down.
 * (+) tid, the program's Task ID;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j down(int tid) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Changes the current stack frame of tid to the given frame number.
 * (+) tid, the program's Task ID;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j frame(int tid, int frame) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Gets information about the current breakpoints int tid.
 * (+) tid, the program's Task ID;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j info_break(int tid) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Gets information about the current display expressions in tid.
 * (+) tid, the program's Task ID;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j info_display(int tid) throws NullPointerException, FiddleException, DebuggerException;






// ---- Process Thread Methods --->>> 

/**
 * Selects thread corresponding to the given threadID within 
 * the Task's ID context as the target for the next debugging 
 * commands.
 * (+) tid, the program's Task ID;
 * (+) threadID, the thread's ID;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j thread(int tid, int threadID) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Gets information about all the threads in tid.
 * (+) tid, the program's Task ID;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j info_threads(int tid) throws NullPointerException, FiddleException, DebuggerException;







// ---- Miscellaneous Methods --->>> 

/**
 * Sets the standard I/O tty to the specified tty device
 * for the program being debugged.
 * (+) tid, the program's Task ID;
 * (+) tty, the tty name to redirect the program's output;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j tty(int tid, String tty_dev) throws NullPointerException, FiddleException, DebuggerException;

/**
 * Sends a command directly to the debugger associated
 * to the process' ID.
 * (+) tid, the program's Task ID;
 * (+) cmd, the command line to send;
 * (-) the JAVA image of the Tkout structure.
 */
	 public native Tkout_j sendTo(int tid, String cmd) throws NullPointerException, FiddleException, DebuggerException;

}
