/* *****************************************************************
 *
 *  $Id: do_rep.c,v 1.1 2002/08/31 15:23:00 rja Exp $
 *
 *  Copyright (c) 2001 by Dep.Informatica of the FCT/UNL
 *  Time-stamp: <02/08/26 12:45:12 rja>
 *
 *  Ricardo Anastacio [jml] <rja@asc.di.fct.unl.pt>
 *
 *  Parallel and Distributed Processing Group
 *  Departamento de Informatica
 *  Faculdade de Ciencias e Tecnologia
 *  Universidade Nova de Lisboa
 *
 * -----------------------------------------------------------------
 *
 *  Description:
 *    Processes the rep structure in the tkout and into a fiddle_j object.
 * -----------------------------------------------------------------
 */

#include "do_rep.h"

/* #################################################################
 *		#DEFINES, TYPES and GLOBAL (PRIVATE) VARIABLES
 * ################################################################# */

PRIVATE JNIEnv *env;
PRIVATE tkout_t *tk;
PRIVATE repOb_t rObj;


/* #################################################################
 *		PRIVATE functions
 * ################################################################# */


#ifdef EBUG
PRIVATE void passo(float f) {
  fprintf(stderr, "do_rep->Passo %.2f.\n", f);
  fflush(stderr);
}
#endif





/* #################################################################
 *		PUBLIC functions
 * ################################################################# */


/* ###################### The reply-specific functions ############# */

/* FILE */
PUBLIC int process_rep_file(void *token, void *no_args) {
	jclass      cls;
	jmethodID   mid;
	jobject     rfile;

	rfile_t *data = (rfile_t *) token;
	jint    tid   = data->tid;

	cls = (*env)->FindClass(env, FILE_J_SIG);
	if(cls == 0) fatalError("process_rep:process_rep_structure:FindClass:File_j\n", -17);
	mid = (*env)->GetMethodID(env, cls, "<init>", "(I)V");
	if(mid == 0) fatalError("process_rep:process_rep_structure:GetMethodID: File_j Constructor\n", -2);
	rfile = (*env)->NewObject(env, cls, mid, tid);

	insertObjInJAVA(env, rfile, rObj->rep);

	return 0;
}



/* BREAK */
PUBLIC int process_rep_break(void *token, void *no_args) {
	jclass      cls;
	jmethodID   mid;
	jobject     rbreak;

	rbreak_t *data = (rbreak_t *) token;
	jint     bpid    = data->bpid;
	jlong    address = (unsigned int) data->address;
	jstring  file    = (*env)->NewStringUTF(env, data->file);
	jint     line    = data->line;

	cls = (*env)->FindClass(env, BREAK_J_SIG);
	if(cls == 0) fatalError("process_rep:process_rep_structure:FindClass:break_j\n", -19);
	mid = (*env)->GetMethodID(env, cls, "<init>", "(IJLjava/lang/String;I)V");
	if(mid == 0) fatalError("process_rep:process_rep_structure:GetMethodID: Break_j Constructor\n", -2);
	rbreak = (*env)->NewObject(env, cls, mid, bpid, address, file, line);

	insertObjInJAVA(env, rbreak, rObj->rep);

	return 0;
}




/* RUN */
PUBLIC int process_rep_run(void *token, void *no_args) {
	jclass      cls;
	jmethodID   mid;
	jobject     rrun;

	rrun_t *data = (rrun_t *) token;
	jstring file = (*env)->NewStringUTF(env, data->file);

	cls = (*env)->FindClass(env, RUN_J_SIG);
	if(cls == 0) fatalError("process_rep:process_rep_structure:FindClass:run_j\n", -20);
	mid = (*env)->GetMethodID(env, cls, "<init>", "(Ljava/lang/String;)V");
	if(mid == 0) fatalError("process_rep:process_rep_structure:GetMethodID: Run_j Constructor\n", -2);
	rrun = (*env)->NewObject(env, cls, mid, file);

	insertObjInJAVA(env, rrun, rObj->rep);

	return 0;
}




/* CONTINUE */
PUBLIC int process_rep_continue(void *token, void *no_args) {
  // Nothing to do!!!
  
	return 0;
}





/* DELETE */
PUBLIC int process_rep_delete(void *token, void *no_args) {
  // Nothing to do!!!

	return 0;
}




/* LIST */
PUBLIC int process_rep_list(void *token, void *no_args) {
	jclass      cls;
	jmethodID   mid;
	jobject     rlist;

	rlist_t *data = (rlist_t *) token;
	jint    line  = data->line_number;
	jstring text  = (*env)->NewStringUTF(env, data->line_text);

	cls = (*env)->FindClass(env, LIST_J_SIG);
	if(cls == 0) fatalError("process_rep:process_rep_structure:FindClass:List_j\n", -21);
	mid = (*env)->GetMethodID(env, cls, "<init>", "(ILjava/lang/String;)V");
	if(mid == 0) fatalError("process_rep:process_rep_structure:GetMethodID: List_j Constructor\n", -2);
	rlist = (*env)->NewObject(env, cls, mid, line, text);

	insertObjInJAVA(env, rlist, rObj->rep);

	return 0;
}



/* TIDS */
PUBLIC int process_rep_tids(void *token, void *no_args) {
	jclass      cls;
	jmethodID   mid;
	jobject     rtids;

	rtids_t *data    = (rtids_t *) token;
	jint    gtid     = data->gtid;
	jint    ltid     = data->ltid;
	jint    tp_pid   = data->tp_pid;
	jint    dbg_pid  = data->dbg_pid;
	jint    attached = data->attached;
	jstring machine  = (*env)->NewStringUTF(env, data->machine);

	cls = (*env)->FindClass(env, TIDS_J_SIG);
	if(cls == 0) fatalError("process_rep:process_rep_structure:FindClass:Tids_j\n", -22);
	mid = (*env)->GetMethodID(env, cls, "<init>", "(IILjava/lang/String;III)V");
	if(mid == 0) fatalError("process_rep:process_rep_structure:GetMethodID: Tids_j Constructor\n", -2);
	rtids = (*env)->NewObject(env, cls, mid, gtid, ltid, machine, tp_pid, dbg_pid, attached);

	insertObjInJAVA(env, rtids, rObj->rep);

	return 0;
}




/* ATTACH */
PUBLIC int process_rep_attach(void *token, void *no_args) {
	jclass      cls;
	jmethodID   mid;
	jobject     rattach;

	rattach_t *data  = (rattach_t *) token;
	jint      tid    = data->tid;
	jint      tp_pid = data->tp_pid;

	cls = (*env)->FindClass(env, ATTACH_J_SIG);
	if(cls == 0) fatalError("process_rep:process_rep_structure:FindClass:Attach_j\n", -23);
	mid = (*env)->GetMethodID(env, cls, "<init>", "(II)V");
	if(mid == 0) fatalError("process_rep:process_rep_structure:GetMethodID: Attach_j Constructor\n", -2);
	rattach = (*env)->NewObject(env, cls, mid, tid, tp_pid);

	insertObjInJAVA(env, rattach, rObj->rep);

	return 0;
}




/* STEP */
PUBLIC int process_rep_step(void *token, void *no_args) {
  // Nothing to do!!!

	return 0;
}





/* NEXT */
PUBLIC int process_rep_next(void *token, void *no_args) {
  // Nothing to do!!!

	return 0;
}



/* FINISH */
PUBLIC int process_rep_finish(void *token, void *no_args) {
	jclass      cls;
	jmethodID   mid;
	jobject     rfinish;

	rfinish_t *data = (rfinish_t *) token;
	jlong    address  = (unsigned int) data->s_address;
	jstring  function = (*env)->NewStringUTF(env, data->s_function);
	jstring  args     = (*env)->NewStringUTF(env, data->s_args);
	jstring  file     = (*env)->NewStringUTF(env, data->s_file);
	jint     line     = data->s_line;
	jint     dbg_var  = data->dbg_var;
	jstring  value    = (*env)->NewStringUTF(env, data->value);

	cls = (*env)->FindClass(env, FINISH_J_SIG);
	if(cls == 0) fatalError("process_rep:process_rep_structure:FindClass:Finish_j\n", -24);
	mid = (*env)->GetMethodID(env, cls, "<init>", "(JLjava/lang/String;Ljava/lang/String;Ljava/lang/String;IILjava/lang/String;)V");
	if(mid == 0) fatalError("process_rep:process_rep_structure:GetMethodID: Finish_j Constructor\n", -2);
	rfinish = (*env)->NewObject(env, cls, mid, address, function, args, file, line, dbg_var, value);

	insertObjInJAVA(env, rfinish, rObj->rep);

	return 0;
}



/* SENDTO */
PUBLIC int process_rep_sendto(void *token, void *no_args) {
	jclass      cls;
	jmethodID   mid;
	jobject     rsendto;

	rsendto_t *data  = (rsendto_t *) token;
	jstring   output = (*env)->NewStringUTF(env, data->output);

	cls = (*env)->FindClass(env, SENDTO_J_SIG);
	if(cls == 0) fatalError("process_rep:process_rep_structure:FindClass:Sendto_j\n", -25);
	mid = (*env)->GetMethodID(env, cls, "<init>", "(Ljava/lang/String;)V");
	if(mid == 0) fatalError("process_rep:process_rep_structure:GetMethodID: Sendto_j Constructor\n", -2);
	rsendto = (*env)->NewObject(env, cls, mid, output);

	insertObjInJAVA(env, rsendto, rObj->rep);

	return 0;
}


/* DISPLAY */
PUBLIC int process_rep_display(void *token, void *no_args) {
	jclass      cls;
	jmethodID   mid;
	jobject     rdisplay;

	rdisplay_t *data = (rdisplay_t *) token;
	jint        dispid     = data->dispid;
	jboolean    enabled    = (data->enabled ? JNI_TRUE : JNI_FALSE);
	jstring     expression = (*env)->NewStringUTF(env, data->expression);
	jstring     value      = (*env)->NewStringUTF(env, data->value);
	/* RIC: O valor do enabled no est a ser ignorado e (no) j est a ser passado para cima... */
	/* Confirmar com o jml */
//	jbool      enabled = data->enabled;

	cls = (*env)->FindClass(env, DISPLAY_J_SIG);
	if(cls == 0) fatalError("process_rep:process_rep_structure:FindClass:Display_j\n", -8);
	mid = (*env)->GetMethodID(env, cls, "<init>", "(IZLjava/lang/String;Ljava/lang/String;)V");
	if(mid == 0) fatalError("process_rep:process_rep_structure:GetMethodID: Display_j Constructor\n", -2);
	rdisplay = (*env)->NewObject(env, cls, mid, dispid, enabled, expression, value);

	insertObjInJAVA(env, rdisplay, rObj->rep);

	return 0;
}



/* UNDISPLAY */
PUBLIC int process_rep_undisplay(void *token, void *no_args) {
  // Nothing to do!!!

	return 0;
}




/* SET_VARIABLE */
PUBLIC int process_rep_set_variable(void *token, void *no_args) {
  // Nothing to do!!!

	return 0;
}




/* EVALUATE */
PUBLIC int process_rep_evaluate(void *token, void *no_args) {
	jclass      cls;
	jmethodID   mid;
	jobject     reval;

	revaluate_t *data = (revaluate_t *) token;
	jint      dbg_var = data->dbg_var;
	jstring   value   = (*env)->NewStringUTF(env, data->value);
	
	cls = (*env)->FindClass(env, EVALUATE_J_SIG);
	if(cls == 0) fatalError("process_rep:process_rep_structure:FindClass:Evaluate_j\n", -26);
	mid = (*env)->GetMethodID(env, cls, "<init>", "(ILjava/lang/String;)V");
	if(mid == 0) fatalError("process_rep:process_rep_structure:GetMethodID: Evaluate_j Constructor\n", -2);
	reval = (*env)->NewObject(env, cls, mid, dbg_var, value);

	insertObjInJAVA(env, reval, rObj->rep);

	return 0;
}




/* THREAD */
PUBLIC int process_rep_thread(void *token, void *no_args) {
  // Nothing to do!!!

	return 0;
}


/* DETACH */
PUBLIC int process_rep_detach(void *token, void *no_args) {
	jclass      cls;
	jmethodID   mid;
	jobject     rdetach;

	rdetach_t *data  = (rdetach_t *) token;
	jint      tp_pid = data->tp_pid;
	
	cls = (*env)->FindClass(env, DETACH_J_SIG);
	if(cls == 0) fatalError("process_rep:process_rep_structure:FindClass:Detach_j\n", -27);
	mid = (*env)->GetMethodID(env, cls, "<init>", "(I)V");
	if(mid == 0) fatalError("process_rep:process_rep_structure:GetMethodID: Detach_j Constructor\n", -2);
	rdetach = (*env)->NewObject(env, cls, mid, tp_pid);

	insertObjInJAVA(env, rdetach, rObj->rep);

	return 0;
}




/* KILL */
PUBLIC int process_rep_kill(void *token, void *no_args) {
  // Nothing to do!!!

	return 0;
}



/* SYMBOL_FILE */
PUBLIC int process_rep_symbol_file(void *token, void *no_args) {
  // Nothing to do!!!

	return 0;
}


/* SIGNAL */
PUBLIC int process_rep_signal(void *token, void *no_args) {
  // Nothing to do!!! (Data is returned in the Out of Band structure)

  return 0;
}


/* CALL */
PUBLIC int process_rep_call(void *token, void *no_args) {
  // Call returns the same as evaluate in rep.
  return process_rep_evaluate(token, no_args);
}


/* INFO_LINE */
PUBLIC int process_rep_info_line(void *token, void *no_args) {
	jclass      cls;
	jmethodID   mid;
	jobject     rinfoline;

	rinfoline_t *data                  = (rinfoline_t *) token;
	jint         line                  = data->line;
	jstring      file                  = (*env)->NewStringUTF(env, data->file);
	jstring      start_function        = (*env)->NewStringUTF(env, data->start_function);
	jint         start_function_offset = data->start_function_offset;
	jstring      end_function          = (*env)->NewStringUTF(env, data->end_function);
	jint         end_function_offset   = data->end_function_offset;
	jlong        start_address         = (unsigned int) data->start_address;
	jlong        end_address           = (unsigned int) data->end_address;


	cls = (*env)->FindClass(env, INFOLINE_J_SIG);
	if(cls == 0) fatalError("process_rep:process_rep_structure:FindClass:Infoline_j\n", -28);
	mid = (*env)->GetMethodID(env, cls, "<init>", "(ILjava/lang/String;Ljava/lang/String;ILjava/lang/String;IJJ)V");
	if(mid == 0) fatalError("process_rep:process_rep_structure:GetMethodID: Infoline_j Constructor\n", -2);
	rinfoline = (*env)->NewObject(env, cls, mid, line, file, start_function, start_function_offset, end_function, end_function_offset, start_address, end_address);

	insertObjInJAVA(env, rinfoline, rObj->rep);

	return 0;
}



/* INFO_LINE */
PUBLIC int process_rep_info_program(void *token, void *no_args) {
	jclass      cls;
	jmethodID   mid;
	jobject     rinfoprog;

	rinfoprogram_t *data    = (rinfoprogram_t *) token;
	jint         tp_pid  = data->tp_pid;
	jlong        address = (unsigned int) data->address;
	jstring      why     = (*env)->NewStringUTF(env, data->why);
	jint         bpid    = data->bpid;
	jstring      signal  = (*env)->NewStringUTF(env, data->signal);


	cls = (*env)->FindClass(env, INFOPROGRAM_J_SIG);
	if(cls == 0) fatalError("process_rep:process_rep_structure:FindClass:Infoprogram_j\n", -29);
	mid = (*env)->GetMethodID(env, cls, "<init>", "(IJLjava/lang/String;ILjava/lang/String;)V");
	if(mid == 0) fatalError("process_rep:process_rep_structure:GetMethodID: Infoprogram_j Constructor\n", -2);
	rinfoprog = (*env)->NewObject(env, cls, mid, tp_pid, address, why, bpid, signal);

	insertObjInJAVA(env, rinfoprog, rObj->rep);

	return 0;
}



/* INFO_STACK */
PUBLIC int process_rep_info_stack(void *token, void *no_args) {
	jclass      cls;
	jmethodID   mid;
	jobject     rinfostack;

	rframe_t *data      = (rframe_t *) token;
	jint      frame     = data->frame;
	jlong     address   = (unsigned int) data->address;
	jstring   function  = (*env)->NewStringUTF(env, data->function);
	jstring   arguments = (*env)->NewStringUTF(env, data->arguments);
	jstring   file      = (*env)->NewStringUTF(env, data->file);
	jint      line      = data->line;


	cls = (*env)->FindClass(env, FRAME_J_SIG);
	if(cls == 0) fatalError("process_rep:process_rep_structure:FindClass:Frame_j\n", -30);
	mid = (*env)->GetMethodID(env, cls, "<init>", "(IJLjava/lang/String;Ljava/lang/String;Ljava/lang/String;I)V");
	if(mid == 0) fatalError("process_rep:process_rep_structure:GetMethodID: Frame_j Constructor\n", -2);
	rinfostack = (*env)->NewObject(env, cls, mid, frame, address, function, arguments, file, line);

	insertObjInJAVA(env, rinfostack, rObj->rep);

	return 0;
}




/* UP */
PUBLIC int process_rep_up(void *token, void *no_args) {
  // Up returns the same as info_stack in rep.
  return process_rep_info_stack(token, no_args);
}




/* DOWN */
PUBLIC int process_rep_down(void *token, void *no_args) {
  // Down returns the same as info_stack in rep.
  return process_rep_info_stack(token, no_args);
}




/* FRAME */
PUBLIC int process_rep_frame(void *token, void *no_args) {
  // Frame returns the same as info_stack in rep.
  return process_rep_info_stack(token, no_args);
}



/* INFO_BREAK */
PUBLIC int process_rep_info_break(void *token, void *no_args) {
	jclass      cls;
	jmethodID   mid;
	jobject     rinfobreak;

	rinfobreak_t *data = (rinfobreak_t *) token;
	jint          bpid        = data->bpid;
	jstring       type        = (*env)->NewStringUTF(env, data->type);
	jstring       disp        = (*env)->NewStringUTF(env, data->disp);
	jboolean      enabled     = (data->enabled ? JNI_TRUE : JNI_FALSE);
	jlong         address     = (unsigned int) data->address;
	jstring       function    = (*env)->NewStringUTF(env, data->function);
	jstring       file        = (*env)->NewStringUTF(env, data->file);
	jint          line        = data->line;
	jint          hit_counter = data->hit_counter;
	jboolean      ign_counter = (data->ign_counter ? JNI_TRUE : JNI_FALSE);
	jstring       condition   = (*env)->NewStringUTF(env, data->condition);

	cls = (*env)->FindClass(env, INFOBREAK_J_SIG);
	if(cls == 0) fatalError("process_rep:process_rep_structure:FindClass:Infobreak_j\n", -31);
	mid = (*env)->GetMethodID(env, cls, "<init>", "(IJIILjava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;ZZ)V");
	if(mid == 0) fatalError("process_rep:process_rep_structure:GetMethodID: Infobreak_j Constructor\n", -2);
	rinfobreak = (*env)->NewObject(env, cls, mid, bpid, address, line, hit_counter, function, file, condition, type, disp, enabled, ign_counter);

	insertObjInJAVA(env, rinfobreak, rObj->rep);

	return 0;
}




/* INFO_DISPLAY */
PUBLIC int process_rep_info_display(void *token, void *no_args) {
  jclass      cls;
  jmethodID   mid;
  jobject     rinfodisp;
  
  rdisplay_t *data = (rdisplay_t *) token;
  jint dispid = data->dispid;
  jboolean enabled = ( data->enabled ? JNI_TRUE : JNI_FALSE );
  jstring expression = (*env)->NewStringUTF(env, data->expression);
  jstring value      = (*env)->NewStringUTF(env, data->value);
  
  cls = (*env)->FindClass(env, DISPLAY_J_SIG);
  if(cls == 0) fatalError("process_rep:process_rep_structure:FindClass:Display_j\n", -8);
  mid = (*env)->GetMethodID(env, cls, "<init>", "(IZLjava/lang/String;Ljava/lang/String;)V");
  if(mid == 0) fatalError("process_rep:process_rep_structure:GetMethodID:Display_j Constructor\n", -2);
  rinfodisp = (*env)->NewObject(env, cls, mid, dispid, enabled, expression, value);

  insertObjInJAVA(env, rinfodisp, rObj->rep);
  
  return 0;
}




/* INFO_THREADS */
PUBLIC int process_rep_info_threads(void *token, void *no_args) {
	jclass      cls;
	jmethodID   mid;
	jobject     rinfothreads;

	rinfothreads_t *data = (rinfothreads_t *) token;
	jboolean active   = ( data->active ? JNI_TRUE : JNI_FALSE );
	jint th_id        = data->th_id;
	jint tp_pid       = data->tp_pid;
	jlong frame       = (unsigned int) data->frame;
	jstring function  = (*env)->NewStringUTF(env, data->function);
	jstring arguments = (*env)->NewStringUTF(env, data->arguments);
	jstring file      = (*env)->NewStringUTF(env, data->file);
	jint line         = data->line;

	cls = (*env)->FindClass(env, INFOTHREADS_J_SIG);
	if(cls == 0) fatalError("process_rep:process_rep_structure:FindClass:Infothreads_j\n", -32);
	mid = (*env)->GetMethodID(env, cls, "<init>", "(ZIIJLjava/lang/String;Ljava/lang/String;Ljava/lang/String;I)V");
	if(mid == 0) fatalError("process_rep:process_rep_structure:GetMethodID: Infothreads_j Constructor\n", -2);
	rinfothreads = (*env)->NewObject(env, cls, mid, active, th_id, tp_pid, frame, function, arguments, file, line);

	insertObjInJAVA(env, rinfothreads, rObj->rep);

	return 0;
}



/* TTY */
PUBLIC int process_rep_tty(void *token, void *no_args) {
  // Nothing to do!!!

	return 0;
}





/* CLIENTS */
PUBLIC int process_rep_clients(void *token, void *no_args) {
	jclass      cls;
	jmethodID   mid;
	jobject     rclients;

	rclients_t *data = (rclients_t *) token;
	jint        pid      = (int) data->pid;
	jstring     filename = (*env)->NewStringUTF(env, data->filename);
	jstring     hostaddr = (*env)->NewStringUTF(env, data->hostaddr);

	cls = (*env)->FindClass(env, CLIENTS_J_SIG);
	if(cls == 0) fatalError("process_rep:process_rep_structure:FindClass:Clients_j\n", -34);
	mid = (*env)->GetMethodID(env, cls, "<init>", "(ILjava/lang/String;Ljava/lang/String;)V");
	if(mid == 0) fatalError("process_rep:process_rep_structure:GetMethodID: Clients_j Constructor\n", -2);
	rclients = (*env)->NewObject(env, cls, mid, pid, filename, hostaddr);

	insertObjInJAVA(env, rclients, rObj->rep);

	return 0;
}




/* ################# The "main" function ############################# */

PUBLIC void process_reply(JNIEnv *envArg, repOb_t rObjArg, tkout_t *tkArg, int (*process_rep_structure)(void *, void *)) {
  jclass    cls;
  jmethodID mid;
  jobject   str;
  jobject   code;
  jobject   data;
  char      buffer[MAX_STR];

#ifdef EBUG
  passo(0);
#endif

  env = envArg;
  rObj = rObjArg;
  tk = tkArg;

  if(tk->rep != NULL) {
	// There's work to do, let's do it then!

#ifdef EBUG
	fprintf(stderr, "tk->rep->str=|%s|, 0x%x\n", tk->rep->str, (unsigned int) tk->rep->str);
	passo(1);
#endif

// The string field from the rep structure
	if(tk->rep->str == NULL) tk->rep->str = strdup("");
	str = (*env)->NewStringUTF(env, tk->rep->str);

#ifdef EBUG
  passo(2);
#endif

// The code field from the rep structure
	cls = (*env)->FindClass(env, CODE_J_SIG);
	if(cls == 0) fatalError("process_rep:FindClass:Code_j\n", -1);
	mid = (*env)->GetMethodID(env, cls, "<init>", "(I)V");
	if(mid == 0) fatalError("process_rep:GetMethodID:Code_j Constructor\n", -2);
	code = (*env)->NewObject(env, cls, mid, tk->rep->code);

#ifdef EBUG
  passo(3);
#endif

// An empty field
	cls = (*env)->FindClass(env, CHAIN_J_SIG);
	if(cls == 0) fatalError("process_rep:FindClass:Chain_j\n", -13);
	mid = (*env)->GetMethodID(env, cls, "<init>", "()V");
	if(mid == 0) fatalError("process_rep:GetMethodID:Chain_j Constructor\n", -2);
	data = (*env)->NewObject(env, cls, mid);
	

#ifdef EBUG
  passo(4);
#endif

// Create the rep object with the str and code values set and an empty
// data (Chain_J) field
	cls = (*env)->FindClass(env, DATA_J_SIG);
	if(cls == 0) fatalError("process_rep:FindClass:Data_j\n", -14);
	mid = (*env)->GetMethodID(env, cls, "<init>", StrCodeChain_sigV(buffer));
	if(mid == 0) fatalError("process_rep:GetMethodID:Data_j Constructor\n", -2);
	rObj->rep = (*env)->NewObject(env, cls, mid, str, code, data);

#ifdef EBUG
	fprintf(stderr, "rObj=0x%x\nrObj->rep=0x%x\nmid=0x%x\nstr=0x%x\ncode=0x%x\ndata=0x%x\n",
			(unsigned int) rObj, (unsigned int) rObj->rep, (unsigned int) mid, (unsigned int) str, (unsigned int) code, (unsigned int) data);
  passo(5);
#endif

// Fill the data field of the rep Object
	if(chain_walk(tk->rep->data, process_rep_structure, NULL) != NULL)
	  fatalError("Error processing rep structure structure. Traversing not completed. Program canceled.", -15);

#ifdef EBUG
  passo(6);
#endif

  } else {
#ifdef EBUG
	passo(7);
#endif
  
// Create an empty rep object
	cls = (*env)->FindClass(env, DATA_J_SIG);
	if(cls == 0) fatalError("process_rep:FindClass:Data_j\n", -14);
	mid = (*env)->GetMethodID(env, cls, "<init>", "()V");
	if(mid == 0) fatalError("process_rep:GetMethodID:Data_j Constructor\n", -2);
	rObj->rep = (*env)->NewObject(env, cls, mid);

#ifdef EBUG
  passo(8);
#endif
  }
	
}
