/* *****************************************************************
 *
 *  $Id: do_rep.h,v 1.1 2002/08/31 15:23:00 rja Exp $
 *
 *  Copyright (c) 2001 by Dep.Informatica of the FCT/UNL
 *  Time-stamp: <02/08/13 19:48:31 rja>
 *
 *  Ricardo Anastacio [jml] <rja@asc.di.fct.unl.pt>
 *
 *  Parallel and Distributed Processing Group
 *  Departamento de Informatica
 *  Faculdade de Ciencias e Tecnologia
 *  Universidade Nova de Lisboa
 *  
 * -----------------------------------------------------------------
 *
 *  Description:
 *    A set of functions to process the rep field from the tkout_t
 * object obtained after calling the fiddle library function.
 * -----------------------------------------------------------------
 */
#ifndef __DO_REP_H_
#define __DO_REP_H_

#include "common.h"
#include "string.h"

/*
 * Processes de rep C structure in the tkout_t
 * structure, parseing it and constructing its JAVA image.
 * (+) env, the JAVA enviroment variable;
 * (+/-) rObj, the structure holding the reply objects;
 * (+) tk, the tkout_t structure to process;
 * (+) process_rep_structure, the function specific that will
 *                 be used to traverse the chain_t structure.
 */
PUBLIC void process_reply(JNIEnv *env, repOb_t rObj, tkout_t *tk, int (*process_rep_structure)(void *, void *));

/* ################### REPLY SPECIFIC FUNCTIONS #################### */

/*
 * Every functions has the same format:
 *
 * For every node in the data field of the rep structure (data_t) this
 * function is aplyed, through the iteration of Chain_walk(...). For
 * every data in the node a corresponding JAVA image is created and
 * enqueued in the data field (chain_j) of the fiddle_j rep object
 * (data_j).
 * (+) token, the data in the node (see chain_walk() );
 * (+) no_args, extra arguments. This is ignored!
 * (-) A success value. If negative the chain_walk function will stop
 * the traversing.
 *
 * int process_rep_structure(void *token, void *no_args);
 */

PUBLIC int process_rep_file(void *token, void *no_args);
PUBLIC int process_rep_break(void *token, void *no_args);
PUBLIC int process_rep_run(void *token, void *no_args);
PUBLIC int process_rep_continue(void *token, void *no_args);
PUBLIC int process_rep_delete(void *token, void *no_args);
PUBLIC int process_rep_list(void *token, void *no_args);
PUBLIC int process_rep_tids(void *token, void *no_args);
PUBLIC int process_rep_attach(void *token, void *no_args);
PUBLIC int process_rep_step(void *token, void *no_args);
PUBLIC int process_rep_next(void *token, void *no_args);
PUBLIC int process_rep_finish(void *token, void *no_args);
PUBLIC int process_rep_sendto(void *token, void *no_args);
PUBLIC int process_rep_display(void *token, void *no_args);
PUBLIC int process_rep_undisplay(void *token, void *no_args);
PUBLIC int process_rep_set_variable(void *token, void *no_args);
PUBLIC int process_rep_evaluate(void *token, void *no_args);
PUBLIC int process_rep_thread(void *token, void *no_args);
PUBLIC int process_rep_detach(void *token, void *no_args);
PUBLIC int process_rep_kill(void *token, void *no_args);
PUBLIC int process_rep_symbol_file(void *token, void *no_args);
PUBLIC int process_rep_signal(void *token, void *no_args);
PUBLIC int process_rep_call(void *token, void *no_args);
PUBLIC int process_rep_info_line(void *token, void *no_args);
PUBLIC int process_rep_info_program(void *token, void *no_args);
PUBLIC int process_rep_info_stack(void *token, void *no_args);
PUBLIC int process_rep_up(void *token, void *no_args);
PUBLIC int process_rep_down(void *token, void *no_args);
PUBLIC int process_rep_frame(void *token, void *no_args);
PUBLIC int process_rep_info_break(void *token, void *no_args);
PUBLIC int process_rep_info_display(void *token, void *no_args);
PUBLIC int process_rep_info_threads(void *token, void *no_args);
PUBLIC int process_rep_tty(void *token, void *no_args);
PUBLIC int process_rep_clients(void *token, void *no_args);


#endif /* __DO_REP_H_ */
