/* ******************************************************************
 *
 *  $Id: Chain_j.java,v 1.1 2002/08/31 15:23:00 rja Exp $
 *
 *  Copyright (c) 2000 by Dep.Informatica of the FCT/UNL
 *  Time-stamp: <02/08/22 14:39:05 rja>
 *
 *  Ricardo Anastacio [jml] <rja@asc.di.fct.unl.pt>
 *
 *  Parallel and Distributed Processing Group
 *  Departamento de Informatica
 *  Faculdade de Ciencias e Tecnologia
 *  Universidade Nova de Lisboa
 *
 * -----------------------------------------------------------------
 *
 *  Description:
 *
 * Class Chain_j is the blueprint of the java object that will
 * represent C's chain_t variable
 * -----------------------------------------------------------------
 */

package unl.fct.di.fiddle_j.util;

import java.util.*;    // For LinkedList, NoSuchElementException, ...


public class Chain_j extends LinkedList {


/* ########## CONSTRUCTORS ########## */

/**
 * Constructs a Chain_j object.
 */
	public Chain_j() {
		super();
	}

/**
 * Constructs a Chain_j object by copy.
 */
	 public Chain_j(Chain_j c) {
		  super(c);
	 }

/**
 * Constructs a Chain_j object with a starting element.
 */
 	public Chain_j(Object obj) {
 		super();
 		this.enqueue(obj);
 	}


/* ######### METHODS INHERITED FROM CLASS java.util.LinkedList ########### */
/*
 *  public void clear();
 *  public Object clone();
 *  public boolean contains(Object o);
 *  public Object get(int index);
 *  public Object getFirst();
 *  public Object getLast();
 *  public int indexOf(Object o);
 *  public int lastIndexOf(Object o);
 *  public ListIterator listIterator(int index);
 *  public Object remove(int index);
 *  public boolean remove(Object o);
 *  public Object[] toArray();
 *  public Object[] toArray(Object[] a);
 */


/* ######### METHODS INHERITED FROM CLASS java.util.AbstractSequentialList ########### */
/*
 *  public Iterator iterator();
 */
	

/* ######### METHODS INHERITED FROM CLASS java.util.AbstractList ########### */
/*
 *  public boolean equals(Object o);
 *  public int hashCode();
 *  public ListIterator listIterator(int index);
 *  protected void removeRange(int fromIndex, int toIndex);
 *  public List subList(int fromIndex, int toIndex);
 */


/* ######### METHODS INHERITED FROM CLASS java.util.AbstractCollection ########### */
/*
 *  public boolean containsAll(Collection c);
 *  public boolean isEmpty();
 *  public boolean removeAll(Collection c);
 *  public boolean retainAll(Collection c);
 *  public String toString();
 */


/* ######### METHODS INHERITED FROM CLASS java.lang.Object ######### */
/*
 *  protected void finalize() throws Throwable;
 *  public final void wait(long timeout, int nanos) throws InterruptedException;
 *  public final void wait(long timeout) throws InterruptedException;
 *  public final void wait() throws InterruptedException;
 *  public final void notifyAll();
 *  public final void notify();
 *  public final Class getClass();
 */

  


/* ######### PRIVATE METHODS ######### */

/*
  Ric Pelos vistos nao existe maneira de esconder estes metodos... O que fazer com eles????
  private void add(int index, Object element);
  private boolean add(Object o);
  private boolean addAll(Collection c);
  private boolean addAll(int index, Collection c);
  private void addFirst(Object o);
  private void addLast(Object o);
  private Object set(int index, Object element);
  private int size();
*/


/* ######### PUBLIC METHODS ######### */


/**
 * Returns a String representation of the Chain_j object
 * and all of its contents.
 */
	 public String toString() {
		  StringBuffer aux = new StringBuffer();
		  int i, length;
		  
		  length = length();
		  aux.append("Chain_j object contents:\n\tlength:").append(length);
		  if(length != 0) {
				aux.append("\n\tdata[0]:").append(this.first().toString());
				for(i=0; i < length-1; i++)
					 aux.append("\n\tdata[").append(i+1).append("]:").append(next(i));
		  }
		  
		  return new String(aux);
	 }


/**
 * Returns the length of the chain.
 */
	public int length() {
		return this.size();
	}


/**
 * Returns the first object in the chain.
 */
	public Object first() {
		return this.getFirst();
	}


/**
 * Returns the last object in the chain.
 */
	public Object last() throws NoSuchElementException {
		return this.getLast();
	}


/**
 * Returns the next object in line after lastIndex.
 */
	public Object next(int lastIndex) throws IndexOutOfBoundsException {
		return this.get(lastIndex + 1);
	}


/**
 * Returns the object previous to the one in lastIndex.
 */
	public Object prev(int lastIndex) throws IndexOutOfBoundsException {
		return this.get(lastIndex - 1);
	}






/**
 * Inserts the object data at the tail of this chain.
 */
	public synchronized int enqueue(Object data) {
		addLast(data);
		notifyAll();
		return this.length()-1;
	}


/**
 * Removes the object at the head of the chain.
 * Returns the removed object.
 */
	public synchronized Object dequeue() throws NoSuchElementException {
		return super.removeFirst();
	}


/**
 * Tests all the nodes in the chain applying the method walker
 * of Chain_walker to each. Removes the first node to which the 
 * walker method returns positive. The removed data is returned.
 * If the walker method returns negative at any point it will 
 * stop the traversing and return null.
 * It leaves all the nodes untouched to which walker returns 0.
 */
	public synchronized Object dequeue_cond(Chain_walker walker_obj, Object args) {
		int i, length, ret_val;
		Object data;

		length = this.length();
		while(true) {
			 try {
				  data = this.first();
				  i = 0;
			 }
			 catch(NoSuchElementException nsee) {
				  return null;  // There's no data in the chain.
			 }

			 while((ret_val = walker_obj.walker(data, args)) == 0) {
				  try {
						i++;
						data = this.get(i);
				  }
				  catch(IndexOutOfBoundsException ioobe) { 
// There's no node in the chain that statisfies the walker's criterium
// we'll wait 'til someone puts anything new to the chain.
						try {
							 Thread.currentThread().wait();
						}
						catch(InterruptedException ie) { }
				  }
			 }		 

			 if(ret_val < 0) {
				  return null;
			 }

			 if(ret_val > 0) {
				  return remove(i);
			 }
		}

	}



/**
 * Traverses the chain from head to tail aplying the walker method of
 * the Chain_walker interface removing all the objects to which walker
 * returns a positive number. Leaving intact those to which walker
 * returns zero.
 * Returns -1 if the resulting chain is empty.
 * Returns the index of the last node visited otherwise.
 */
	public synchronized int prune(Chain_walker walker_obj, Object args) {
		int i, ret_val;
		Object data;

		try {
			data = this.first();
			i = 0;
		}
		catch(NoSuchElementException exc) {
			return -1;  // There's no data in the chain. Prune ended with an empty chain!
		}

		while(i < this.length()) {
			while( (ret_val = walker_obj.walker(data, args)) > 0) {
				this.remove(i);
				try {
					data = this.get(i);
				}
				catch(IndexOutOfBoundsException exc) {
					return i-1;  // There's no more data in the chain.
				}

			}
		
			if(ret_val < 0)  
				return 0;
			
			try {
				i++;
				data = this.get(i);
			}
			catch(IndexOutOfBoundsException exc) {
				return i-1; // We arrived at the end of the chain!
			}

		}

		return i-1; // Strange we ended up here...

	}

/**
 * Appends chain2 to this chain...
 */
	public synchronized void merge(Chain_j chain2) {
		ListIterator li;
		Object data;
		Chain_j copy;
		
		copy = (Chain_j) chain2.clone();
		try {
			li = copy.listIterator(0);
			
			while(li.hasNext()) {
				data = li.next();
				super.addLast(data);
			}
			notifyAll();
		}
		catch(IndexOutOfBoundsException exc1) {
			// chain2 is an empty chain! Nothing to do.
		}
	}


/**
 * Traverses this chain from head to tail applying the method walker
 * of the Chain_walker interface to each object within the chain.
 * Returns the index of the node where the traversing stopped.
 * Returns -1 if the chain is empty.
 */
	public synchronized int walk(Chain_walker walker_obj, Object args) {
		int i, length;
		Object data;

		length = this.length();
		if(length == 0)
			 return -1;

		data = this.first();
		if(walker_obj.walker(data, args)<0)
			 return 0;

		for(i=0; i < length-1; ) {
			data = this.next(i);
			i++;
			if(walker_obj.walker(data, args)<0)
				 break;
		}
		
		return i;
	}


/**
 * Traverses this chain from tail to head applying the method walker
 * of the Chain_walker interface to each object within this chain.
 * Returns the index of the node where the traversing stopped.
 */
	public synchronized int rwalk(Chain_walker walker_obj, Object args) {
		int i, length;
		Object data;

		length = this.length();
		data = this.last();
		if(walker_obj.walker(data, args)<0)
			 return length-1;

		for(i=length-1; i>0; ) {
			data = this.prev(i);
			i--;
			if(walker_obj.walker(data, args)<0)
				 break;
		}
		
		return i;
	}


/**
 * Find the first node in chain for which the method walker_cmp of 
 * the Chain_walker interface returns 0.
 * Returns the index of the node where the traversing stopped.
 * Returns -1 if no corresponding node was found.
 */
	public synchronized int find(Object what) {
		 return indexOf(what);
	}


 


}


