/* *****************************************************************
 *
 *  $Id: test_output.c,v 1.2 2002/07/07 22:38:34 jml Exp $
 *
 *  Copyright (c) 1999 by Dep.Informatica of the FCT/UNL
 *  Time-stamp: <02/07/07 20:10:38 jml>
 *
 *  Joao Lourenco <Joao.Lourenco@di.fct.unl.pt>
 *
 *  Parallel and Distributed Processing Group
 *  Departamento de Informatica
 *  Faculdade de Ciencias e Tecnologia
 *  Universidade Nova de Lisboa
 *
 * -----------------------------------------------------------------
 *
 *  Description:
 *
 * -----------------------------------------------------------------
 */


#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "chain.h"
#include "f0s_tkout.h"
#include "f0s_error.h"
#include "error.h"
#include "test_output.h"
#include "config.h"


#define FIELD_AND_TYPE(major,type)		\
		type *node = (type *) ((major == NULL)	\
				? NULL : chain_first_data (major->data))
#define FIELD(field)				node->field

/* #################################################################
 *		Local functions
 * ################################################################# */


#define F01_FORMAT_STRING   "%10s %6s %9s %9s"
#define F01_FORMAT_INT      "%10d %6c %9d %9d"

#define F2_FORMAT_STRING   "%9s %s"
#define F2_FORMAT_INT      "%9d %s"


PRIVATE int print_tids_line (void *obj, void *args)
{
	char att;
	rtids_t *d = (rtids_t *)obj;

	switch (d->attached) {
	case -1: att = '-'; break;
	case 0:  att = 'n'; break;
	case 1:  att = 'y'; break;
	default: assert (0);
	}
	printf (F01_FORMAT_INT, d->gtid, att, d->tp_pid, d->dbg_pid);
#if !(f0s || f0m)
	printf (F2_FORMAT_INT, d->ltid, d->machine);
#endif
	puts ("");
	return 0;
}


PRIVATE void rep_tids (const data_t *rep)
{
    printf (F01_FORMAT_STRING, "TID", "ATT", "TP_PID", "LLD_PID");
#if !(f0s || f0m)
	printf (F2_FORMAT_STRING, "L_TID", "MACHINE");
#endif
	puts ("");
	if (rep->data != NULL)
		chain_walk (rep->data, print_tids_line, NULL);
}


#if f2m | f3m
#define CLIENTS_FORMAT_HEADER   "%6s  %28s   %s\n"
#define CLIENTS_FORMAT_NODE     "%7d %29s  %s\n"

PRIVATE int print_clients_line (void *obj, void *args)
{
	rclients_t *d = (rclients_t *)obj;

	printf (CLIENTS_FORMAT_NODE, d->pid, d->hostaddr, d->filename);
	return 0;
}

PRIVATE void rep_clients (const data_t *rep)
{
	puts ("Curent Fiddle clients");
    printf (CLIENTS_FORMAT_HEADER, "PID", "HOSTADDR/PORT", "FILENAME");
	if (rep->data != NULL)
		chain_walk (rep->data, print_clients_line, NULL);
}
#endif

PRIVATE void rep_kill (const data_t *rep)
{
	printf ("==> Process was killed\n");
}

PRIVATE void rep_attach (const data_t *rep)
{
	FIELD_AND_TYPE (rep, rattach_t);
	printf ("==> Attached to PID %d\n",
			FIELD (tp_pid));
	printf ("    New Target Identifier (TID) = %d\n",
			FIELD (tid));
}

PRIVATE void rep_detach (const data_t *rep)
{
	FIELD_AND_TYPE (rep, rdetach_t);
	printf ("==> Detaching from program: PID %d\n",
			FIELD (tp_pid));
}
 void rep_symbol_file (const data_t *rep)
{
/* 	printf ("==> Symols loaded from \"%s\"\n", */
/* 			rep->data.r_symbolfile.file); */
	printf ("==> Symols loaded\n");
}

PRIVATE void rep_file (const data_t *rep)
{
	FIELD_AND_TYPE (rep, rfile_t);
/* 	printf ("==> New file loaded from \"%s\"\n", */
/* 			rep->data.r_file.file); */
	printf ("==> New file loaded\n");
	printf ("    New Target Identifier (TID) = %d\n",
			FIELD (tid));
}

PRIVATE void	rep_run (const data_t *rep)
{
	/* Nothing */
}

PRIVATE void	rep_step (const data_t *rep)
{
	/* Nothing */
}

PRIVATE void	rep_next (const data_t *rep)
{
	/* Nothing */
}

PRIVATE void	rep_continue (const data_t *rep)
{
	/* Nothing */
}

PRIVATE void	rep_finish (const data_t *rep)
{
#define AF(f)	node->f == NULL ? "" : node->f

	FIELD_AND_TYPE (rep, rfinish_t);
	printf ("==> 0x%x in %s (%s) at %s:%d\n",
			(unsigned)FIELD (s_address),
			AF (s_function), AF (s_args), AF (s_file), FIELD (s_line));
	printf ("    Value returned is $%d = {%s}\n",
			FIELD (dbg_var), AF (value));
}

PRIVATE void	rep_call_void (const data_t *rep)
{
	/* Nothing */
}

PRIVATE void	rep_break (const data_t *rep)
{
	FIELD_AND_TYPE (rep, rbreak_t);
	printf ("==> Breakpoint %d at 0x%x: file \"%s\", line %d.\n",
			FIELD (bpid),
			(unsigned) FIELD (address),
			FIELD (file),
			FIELD (line));
}

PRIVATE void	rep_break_no_source (const data_t *rep)
{
	FIELD_AND_TYPE (rep, rbreak_t);
	printf ("==> Breakpoint %d at 0x%x\n",
			FIELD (bpid),
			(unsigned) FIELD (address));
}

PRIVATE void	rep_delete (const data_t *rep)
{
	printf ("==> Breakpoint deleted\n");
}

PRIVATE void	rep_tty (const data_t *rep)
{
	/* Nothing */
}



PRIVATE int print_list_line (void *obj, void *args)
{
	rlist_t *l = (rlist_t *)obj;
	if (l != NULL) {
		printf ("==> %4d: ", l->line_number);
		if (l->line_text != NULL)
			printf ("%s", l->line_text);
		puts ("");
	}
	return 0;
}

PRIVATE void	rep_list (const data_t *rep)
{
	if (rep->data != NULL)
		chain_walk (rep->data, print_list_line, NULL);
}


PRIVATE void	rep_info_line (const data_t *rep)
{
	FIELD_AND_TYPE (rep, rinfoline_t);
	printf ("==> Line %d of \"%s\" starts at address 0x%x <%s",
			FIELD (line),
			FIELD (file),
			(unsigned) FIELD (start_address),
			FIELD (start_function));
	if (FIELD (start_function_offset) > 0)
		printf ("+%d",
				FIELD (start_function_offset));
	printf ("> and ends at 0x%x <%s",
			(unsigned) FIELD (end_address),
			FIELD (end_function));
	if (FIELD (end_function_offset) > 0)
		printf ("+%d", FIELD (end_function_offset));
	printf (">.\n");
}

PRIVATE void	rep_info_line_no_code (const data_t *rep)
{
	FIELD_AND_TYPE (rep, rinfoline_t);
	printf ("==> Line %d of \"%s\" is at address 0x%x <%s",
			FIELD (line),
			FIELD (file),
			(unsigned) FIELD (start_address),
			FIELD (start_function));
	if (FIELD (start_function_offset) > 0)
		printf ("+%d",
				FIELD (start_function_offset));
	printf ("> but contains no code.\n");
}

PRIVATE void	rep_info_program (const data_t *rep)
{
	FIELD_AND_TYPE (rep, rinfoprogram_t);
	printf ("\n\
==> Target process %d is stopped at address 0x%x.\n\
    It stopped ",
			FIELD (tp_pid),
			(unsigned) FIELD (address));
	if (strcmp (FIELD (why), "breakpoint") == 0)
		printf ("at breakpoint %d.\n", FIELD (bpid));
	else if (strcmp (FIELD (why), "signal") == 0)
		printf ("with signal %s.\n",
				FIELD (signal));
	else if (strcmp (FIELD (why), "being stepped") == 0)
		printf ("after being stepped.\n");
	else
		printf ("I don't know why!!!!.\n");
}



PRIVATE int print_frame_line (void *obj, void *args)
{
	rframe_t *f = (rframe_t *)obj;
	if (f != NULL) {
		printf ("==> Frame #%d ", f->frame);
		if (f->address != NULL)
			printf ("0x%x in ", (unsigned) f->address);
		if (f->function != NULL) {
			printf ("%s (", f->function);
			if (f->arguments != NULL) 
				printf ("%s", f->arguments);
			printf (") ");
		}
		if (f->file != NULL)
			printf ("at %s:%d", f->file, f->line);
		puts ("");
	}
	return 0;
}

PRIVATE void	rep_info_stack (const data_t *rep)
{
	if (rep->data != NULL)
		chain_walk (rep->data, print_frame_line, NULL);
}



PRIVATE int print_break_line (void *obj, void *args)
{
	rinfobreak_t *b = (rinfobreak_t *)obj;
	if (b != NULL) {
		if (strstr (b->type, "breakpoint") != NULL) {
/*
123412345678901234512345123412345678901
1   breakpoint     keep y   0x08048593 in main at Infinito.c:38
*/
			printf ("    %-3d %-14s %-4s %-3c 0x%x in %s()",
					b->bpid,
					b->type,
					b->disp,
					b->enabled ? 'y' : 'n',
					(unsigned) b->address,
					b->function);
			if (b->file != NULL) 
				printf (" at %s:%d", b->file, b->line);
			if (b->condition != NULL  && strlen (b->condition) > 0)
				printf ("\n\tstop only if %s", b->condition);
		}
		else if (strstr (b->type, "watchpoint") != NULL)
			/*
123412345678901234512345123412345678901
1   hw watchpoint  keep y  str == 0
*/
			printf ("    %-3d %-14s %-4s %-3c %s",
					b->bpid,
					b->type,
					b->disp,
					b->enabled ? 'y' : 'n',
					b->condition);
		if (b->hit_counter != -1)
			printf ("\n\tnumber of hits %d", b->hit_counter);
		if (b->ign_counter != -1)
			printf ("\n\tignore next hits %d", b->ign_counter);
		puts ("");
	}
	return 0;
}

PRIVATE void	rep_info_break (const data_t *rep)
{
		puts ("\
==> Num Type           Disp Enb Address    What");
	if (rep->data != NULL) 
		chain_walk (rep->data, print_break_line, NULL);
}

PRIVATE void	rep_set_variable (const data_t *rep)
{
	/* Nothing */
}


PRIVATE void	rep_evaluate (const data_t *rep)
{
	FIELD_AND_TYPE (rep, revaluate_t);
	printf ("==> $%d = %s\n",
			FIELD (dbg_var), FIELD (value));
}


PRIVATE int	print_locals_line (void *obj, void *args)
{
	rinfolocals_t *d = (rinfolocals_t *)obj;
	printf ("    %s = %s\n", d->var, d->value);
	return 0;
}


PRIVATE void	rep_info_locals (const data_t *rep)
{
	puts ("==> Local Variables / Arguments");
	if (rep->data != NULL)
		chain_walk (rep->data, print_locals_line, NULL);
}


PRIVATE int print_display_line (void *obj, void *args)
{
	rdisplay_t *d = (rdisplay_t *)obj;
	if (d != NULL) {
		printf ("==> %2d: ", d->dispid);
		if (d->enabled != -1)
			printf (" %c ", d->enabled ? 'y' : 'n');
		printf (" (%s)", d->expression);
		if (d->value != NULL)
			printf (" = %s", d->value);
		puts ("");
	}
	return 0;
}

PRIVATE void	rep_display_expression (const data_t *rep)
{
	/* Nothing */
}

PRIVATE void	rep_info_display (const data_t *rep)
{
	puts ("==> Num Enb Expression");
	if (rep->data != NULL)
		chain_walk (rep->data, print_display_line, NULL);
}

PRIVATE void	rep_undisplay (const data_t *rep)
{
	/* Nothing */
}


PRIVATE int print_thread_line (void *obj, void *args)
{
	rinfothreads_t *d = (rinfothreads_t *)obj; 
	if (d != NULL) {
		printf ("==> %c %d Thread %5d  ",
				d->active ? '*' : ' ', d->th_id, d->tp_pid);
		if (d->frame)
			printf ("0x%x in ", (unsigned)d->frame);
		if (d->function != NULL) 
			printf ("%s ", d->function);
		if (d->arguments != NULL) 
			printf ("(%s) ", d->arguments);
		if (d->file != NULL) {
			printf ("at %s:%d", d->file, d->line);
		}
		puts ("");
	}
	return 0;
}

PRIVATE void	rep_info_threads (const data_t *rep)
{
	if (rep->data != NULL)
		chain_walk (rep->data, print_thread_line, NULL);
}

PRIVATE int print_new_thread_line (void *obj, void *args)
{
	onewthread_t *d = (onewthread_t *)obj;
	if (d != NULL) 
		printf ("===> [New thread %d]\n",
				d->id);
	return 0;
}

PRIVATE void	rep_thread (const data_t *rep)
{
	/* Nothing */
}

PRIVATE void	rep_sendto (const data_t *rep)
{
	FIELD_AND_TYPE (rep, rsendto_t);
	printf("\n\
SENDTO: output = \n\
-----------------------------------------------------------------\n\
%s-----------------------------------------------------------------\n",
		   FIELD (output));
}


/* #################################################################
 *		Exported functions
 * ################################################################# */

/* =================================================================
 *		Processing REPLY
 * ================================================================= */


/* -----------------------------------------------------------------
 * Function:    void test_reply (IN const data_t *rep)
 *
 * Description: Intepret the reply field and print the relevant data
 *
 * Returns:     Nothing
 */
void test_reply (const data_t *rep)
{
	if (rep == NULL) return;
		
	switch (rep->code) {
	case E_UNKNOWN_CMD_CODE:
		break;
	case R_TIDS:
		rep_tids (rep);
		break;
#if f2m | f3m
	case R_CLIENTS:
		rep_clients (rep);
		break;
#endif
	case R_KILL:
		rep_kill (rep);
		break;
	case R_ATTACH:
		rep_attach (rep);
		break;
	case R_DETACH:
		rep_detach (rep);
		break;
	case R_SYMBOL_FILE:
		rep_symbol_file (rep);
		break;
	case R_FILE:
		rep_file (rep);
		break;
	case R_RUN:
		rep_run (rep);
		break;
	case R_STEP:
		rep_step (rep);
		break;
	case R_NEXT:
		rep_next (rep);
		break;
	case R_CONTINUE:
		rep_continue (rep);
		break;
	case R_FINISH:
		rep_finish (rep);
		break;
	case R_CALL_VOID:
		rep_call_void (rep);
		break;
	case R_BREAK:
		rep_break (rep);
		break;
	case R_BREAK_NO_SOURCE:
		rep_break_no_source (rep);
		break;
	case R_DELETE:
		rep_delete (rep);
		break;
	case R_SET_VARIABLE:
		rep_set_variable (rep);
		break;
	case R_EVALUATE:
		rep_evaluate (rep);
		break;
	case R_INFO_LOCALS:
		rep_info_locals (rep);
		break;
	case R_DISPLAY_EXPRESSION:
		rep_display_expression (rep);
		break;
	case R_UNDISPLAY:
		rep_undisplay (rep);
		break;
	case R_TTY:
		rep_tty (rep);
		break;
	case R_LIST:
		rep_list (rep);
		break;
	case R_INFO_LINE:
		rep_info_line (rep);
		break;
	case R_INFO_LINE_NO_CODE:
		rep_info_line_no_code (rep);
		break;
	case R_INFO_PROGRAM:
		rep_info_program (rep);
		break;
	case R_INFO_STACK:
		rep_info_stack (rep);
		break;
	case R_INFO_BREAK:
		rep_info_break (rep);
		break;
	case R_INFO_DISPLAY:
		rep_info_display (rep);
		break;
	case R_INFO_THREADS:
		rep_info_threads (rep);
		break;
	case R_THREAD:
		rep_thread (rep);
		break;
	case R_SENDTO:
		rep_sendto (rep);
		break;
	default:
		printf ("\ntest_reply()---Unexpected code = %d\n\ttext = %s\n",
				rep->code, rep->str == NULL ? "" : rep->str);
	}
}


/* =================================================================
 *		Processing ALWAYS
 * ================================================================= */

PRIVATE void	oob_display (const data_t *oob)
{
	if (oob->data != NULL)
		chain_walk (oob->data, print_display_line, NULL);
}

PRIVATE void	oob_new_thread (const data_t *oob)
{
	if (oob->data != NULL)
		chain_walk (oob->data, print_new_thread_line, NULL);
}

PRIVATE void	oob_bphit (const data_t *oob)
{
	FIELD_AND_TYPE (oob, obphit_t);
	printf ("\n\
===> Breakpoint %d hited, \"%s (%s)\" at \"%s:%d\"\n",
			FIELD (bpid), FIELD (function), FIELD (args),
			FIELD (file), FIELD (line));
}

PRIVATE void	oob_signal (const data_t *oob)
{
	FIELD_AND_TYPE (oob, osignal_t);
	printf ("\n\
===> Program received signal %s\n", FIELD (name));
}

PRIVATE void	oob_switch_to_thread (const data_t *oob)
{
	FIELD_AND_TYPE (oob, othread_t);
	printf ("==> [Switching to Thread %d]\n",  FIELD (pid));
}




/* -----------------------------------------------------------------
 * Function:    void test_oob_2 (IN const void *obj, IN void *args)
 *
 * Description: Intepret tone oob field and print the relevant
 *				data.  'args' is ignored.
 *
 * Returns:     Always '0'
 */
PRIVATE int test_oob_2 (void *obj, void *args)
{
	data_t *oob = (data_t *)obj;
	
	switch (oob->code) {
	case E_UNKNOWN_CMD_CODE:
		break;
	case O_DISPLAY:
		oob_display (oob);
		break;
	case O_BPHIT:
		oob_bphit (oob);
		break;
	case O_SIGNAL:
		oob_signal (oob);
		break;
	case O_NEW_THREAD:
		oob_new_thread (oob);
		break;
	case O_SWITCH_TO_THREAD:
		oob_switch_to_thread (oob);
		break;
	default:
		printf ("\ntest_oob_2()---Unexpected code = %d\n\ttext = %s\n",
				oob->code, oob->str == NULL ? "" : oob->str);
	}
	return 0;
}


/* -----------------------------------------------------------------
 * Function:    void test_oob (IN const chain *oob)
 *
 * Description: Intepret the oob field and print the relevant data
 *
 * Returns:     Nothing
 */
void test_oob (chain_t *oob)
{
	if (oob == NULL) return;
	
	chain_walk (oob, test_oob_2, NULL);
}
	

/* =================================================================
 *		Processing POSITION
 * ================================================================= */

/* -----------------------------------------------------------------
 * Function:    void test_position (IN const data_t *pos)
 *
 * Description: Print the position in the source file, if available
 *
 * Returns:     Nothing
 */
void test_position (const data_t *pos)
{
	FIELD_AND_TYPE (pos, pposition_t);

	if (pos == NULL) return;
		
	switch (pos->code) {
	case E_UNKNOWN_CMD_CODE:
		break;
	case P_POSITION:
		printf ("\n\
===> Position:  %s\n\
                line = %d    pos = %d    where = \"%s\"    frame = 0x%x\n",
				FIELD (file), FIELD (line), FIELD (pos),
				FIELD (where), (unsigned) FIELD (frame));
		break;
	default:
		printf ("\ntest_position()---Unexpected code = %d\n\ttext = %s\n",
				pos->code, pos->str == NULL ? "" : pos->str);
	}
}



/* =================================================================
 *		Processing ERROR
 * ================================================================= */

/* -----------------------------------------------------------------
 * Function:    void test_error (IN const data_t *err)
 *
 * Description: Print the error code,messge and CLD output
 *
 * Returns:     Nothing
 */
void test_error (const data_t *err)
{
	const char *msg;
	if (err == NULL) return;	/* No error */

	if (err->code < 0)
		msg = cons_err_msg (err->code);
	else
		msg = f0s_err_msg (err->code);
	printf ("\n==> ERROR %d: %s\n\t%s\n",
			err->code, msg, err->str == NULL ? "" : err->str);
}

/* -----------------------------------------------------------------
 * Function:    void was_error (IN int status)
 *
 * Description: Print the error code and messge
 *
 * Returns:     Nothing
 */
void was_error (int status, const char *usermsg)
{
	const char *msg1;
	const char *msg2;
	if (status == 0) return;	/* No error */
	
	if (status < 0)
		msg1 = cons_err_msg (status);
	else
		msg1 = f0s_err_msg (status);
	if (usermsg == NULL)
		msg2 = "";
	else
		msg2 = usermsg;
	printf ("\n==> ERROR %d: %s\n\t%s", status, msg1, msg2);
}

