/* *****************************************************************
 *
 *  $Id: timer.c,v 1.1.1.1 2002/04/28 22:06:57 jml Exp $
 *
 *  Copyright (c) 1999 by Dep.Informatica of the FCT/UNL
 *  Time-stamp: <99/08/04 15:52:10 jml>
 *
 *  Joao Lourenco <Joao.Lourenco@di.fct.unl.pt>
 *
 *  Parallel and Distributed Processing Group
 *  Departamento de Informatica
 *  Faculdade de Ciencias e Tecnologia
 *  Universidade Nova de Lisboa
 *
 * -----------------------------------------------------------------
 *
 *  Description:
 *
 * -----------------------------------------------------------------
 */

#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>

#include "timer.h"
#include "config.h"

/* #################################################################
 *		#DEFINES, TYPES and VARIABLES
 * ################################################################# */

#define TNULL (Timer *)0
#define ENDOFTIME 31*24*60*60L  /*pelo menos 31 dias (em segundos) */

typedef struct itimerval Timer;

const Timer start = {{0,0},{ENDOFTIME, 0}};


/* #################################################################
 *		PRIVATE functions
 * ################################################################# */

#if 0
static void p_time (const char *msg, const struct timeval *tv)
{
	printf ("%s: (sec=%ld, usec=%ld)\n", msg, tv->tv_sec, tv->tv_usec);
}
#endif


static Time subtract_times (const Timer *start, const Timer *stop)
{
	/* The timer decrements, so do "start - stop" and not the opposite */
    long nsec = start->it_value.tv_sec - stop->it_value.tv_sec;
    long nusec = start->it_value.tv_usec - stop->it_value.tv_usec;
/* 	p_time ("Start", &(start->it_value)); */
/* 	p_time ("Stop", &(stop->it_value)); */
/* 	printf ("Elapsed = %ld\n", nsec * 1000000L + nusec); */
    return (Time) nsec * 1000000L + nusec;
}

/* #################################################################
 *		PUBLIC functions
 * ################################################################# */

/* Start the timer */
int start_timer (void)
{
	return setitimer (ITIMER_REAL, &start, TNULL);
}


/* Stop the timer
 * Return the miliseconds elapsed */
Time stop_timer (void)
{
	Timer stop;
	int err = getitimer (ITIMER_REAL, &stop);
	if (err < 0) return err;
	return subtract_times (&start, &stop);
}


/* Print the time prepending the message */
void print_time (const char *msg, Time t)
{
	printf ("==> Elapsed time for \"%s\" was %1.3f sec  (%ld usec)\n",
			msg, (float)t/1000000L, (long)t);
}
